cat > p.f90 <<\EOR
! Filename:level: icoads-immt5Ximma1.f90:01C                      17 Oct 2013
! Purpose: ICOADS translation program from IMMT5 (stdin) to IMMA1 (stdout)
! Author: S.Lubker
!=============================================================================!
! International Comprehensive Ocean-Atmosphere Data Set (ICOADS)  27 Jun 2013 !
! Filename:level: rdimma1:01D                     Fortran 90/77 program+shell !
! Purpose: Read and print/write IMMA                         Author: S.Lubker !
!=============================================================================!
! Software Revision Information (previous version:  9 May 2013, level 01C):
! No longer recognizes deprecated attachments. Routine {getuid}, to get a
! Unique report ID, was added.  Correction made in the comments below to a
! sentence which previously read "As discussed in Data Representation note
! (2) below, base36 data (ITYPE=3)..." (now reading correctly "ITYPE=2").
!-----------------------------------------------------------------------3456789
! Software documentation for the (modifiable) example program {rdimma} and for
! the (invariant) user-interface routines {prnskp,prnhdr,getrpt,east,getuid,
! minmax,savsum,putrpt,prnrpt,prnsum,init}.
!
! As provided {rdimma}: (a) Prints PROGID (the program name and level).  Also,
! {prnhdr} prints a 5-line header for the unabbreviated record type (i.e., core
! plus all currently defined attms: 1,5-8 and 98-99).  (b) Reads a report (from
! standard input) via a formatted read and calls {getrpt} to extract the fields
! and convert them from characters to numeric values.  This makes the fields
! available in three parallel arrays:
!      CTRUE: characters (each length 1024)  blank       for missing data
!      ITRUE: integers                       NINT(FMISS) for missing data
!      FTRUE: true (floating point) values   FMISS       for missing data
! Additional parallel arrays available within common block IMMA1 provide
! metadata defining the properties of currently defined fields.  These include
! each field's length (ILEN), abbreviation (ABBR), allowable range(s), units
! (FUNITS; missing for character data) and type (ITYPE; 1=numeric, 2=base36,
! 3=character) (see {bdimma} for details).  (c) For the core and attms, a
! PARAMETER statement relates each field abbreviation to an FTRUE (CTRUE,
! ITRUE, etc.) array location; this facilitates usage such as FTRUE(DY),
! to obtain the floating point value for day.  Each element of CTRUE is a
! 1024-character string, but the field length can be used to control printing,
! e.g., PRINT *,CTRUE(DY)(:ILEN(DY)).  (d) Prints the original report, RPT,
! under the aforementioned field headings (constructed using ABBR).  (e) The
! program iterates (to step b) reading and printing reports until end-of-file
! (EOF) is encountered.  Caution: IMPLICIT INTEGER(A-E,G-Z) is used, users may
! need to explicitly type additional variables when modifying this program;
! and deprecated attms 2-4 are no longer recognized (but {rdimma} will continue
! to read any version of IMMA not containing deprecated attachments).
!
! The following optional features are initially deactivated in the Fortran
! code; they may be activated by changing the appropriate Fortran comment
! lines to executable statements, i.e., remove the "!" from column 1:
!
! (a) Activate one or more calls to {prnskp} to suppress printing of the core
! and/or selected attms; {prnskp} is designed to work with the default calls
! to {prnhdr} and {prnrpt} for a reduced-width printout.  Or (b): Deactivate
! the default calls to {prnhdr} and {prnrpt}, and activate one (or more) of
! the alternative calls to {prnhdr} and {prnrpt}.  This provides flexibility
! so that the core or an attm can be printed individually (if only one pair
! of {prnhdr,prnrpt} is activated; alternatively, this can be accomplished by
! activating all but one call to {prnskp}).  Activating multiple alternative
! calls to {prnhdr} and {prnrpt} can be used to arrange the core and attms
! vertically (i.e., core and attms on multiple lines, for a single report).
! Moreover, any contiguous portion of the headers and reports can be printed
! by adjusting the fields that appear in the subroutine calls, e.g., activating
! CALL PRNHDR(STDOUT,ILEN,ABBR,YR,HR) and CALL PRNRPT(STDOUT,ILEN,CTRUE,YR,HR)
! prints just year, month, day, and hour.  Features (a) and (b) provide
! flexibility to print selected portions of the data; however, note that those
! two features were not designed to be used together (e.g., activating {prnskp}
! to skip the core, and {prnhdr,prnrpt} to print the core, will produce blank
! lines).  By default printing is to standard output (STDOUT=6), but optionally
! STDOUT can be changed to another unit number in the calls to {prnhdr,prnrpt}.
!
! Additional notes on using {prnrpt}:  To minimize data volume, IMMA data may
! have a "sparse" record structure, such that attms without any extant data are
! omitted, and the attm structure can vary from report to report.  Sparse data
! printed via the default calls to {prnhdr, prnrpt}, are expanded out into the
! complete IMMA structure.  Or, using {prnskp}, sparse reports are expanded out
! into a reduced print structure.  Alternatively, IMMA data may have a "fixed"
! record structure, such that a fixed number of attms is always present.  In
! either case, it can be determined whether an attm was originally present in
! the input data from ATTI and ATTL: these are printed by {prnrpt} if the attm
! was extant, or blank if it was missing.  ATTC will always reflect the number
! of attms originally present (e.g., regardless how many are omitted from
! printing through the use of {prnskp}).
!
! (c) Activating {east} will transform longitudes stored according to the
! NCDC-variant (obsolescent) convention -179.99 (W) to 180.00 (E), into the
! ICOADS convention 0-359.99 E. This action is applied only to the numeric data
! (ITRUE, FTRUE), and does not take effect on the printed data, unless feature
! (d) is also activated. Activate {getuid} to get the next Unique report ID
! (see feature (h) for details).
!
! (d) Activating {putrpt} will convert all the true values (e.g., longitudes
! transformed via {east}) back into characters for printing and/or writing.
! Specifically, if changes have been made to FTRUE, {putrpt} puts them also
! into ITRUE, CTRUE, and RPT.  Then the changes can be printed, by leaving
! {prnrpt} statement(s) in operation, or written, by activating (instead of
! {prnrpt}) the adjacent WRITE statement.  If both {prnrpt} and the WRITE
! statement are activated, each report will be output twice, possibly with
! differences in format (which provides a means to see how the two different
! output methods work).  For example, note that {prnskp} has no effect on the
! WRITE statement, in contrast to its effect on {prnrpt}.  Reports with a
! sparse record structure will be output when {putrpt} and the WRITE statement
! are used, and ATTC will reflect the output attm count.  Reports with a fixed
! record structure can be correctly output when {prnrpt} is used, with these
! caveats: (1) ATTI and ATTL must be set, when the attm data are entirely
! missing, or they will be printed blank; (2) ATTC will be set incorrectly if
! {prnskp} is used, but any of the attm data are extant.
!
! (e) Activate calls: to {minmax}, to set any "extreme" true values to error
! value FERR (by default equal to the missing value FMISS); to {savsum}, to
! save summary information for each field; and to {prnsum}, to print the
! following summary information on unit 10 (using field abbreviations, ABBR):
!      -- A summary of the numbers (and percentages) of each field that are
!         extant, missing, or erroneous.  Erroneous data are also counted
!         as a subset of the missing category.  Any field (or attm) entirely
!         missing in the input data is omitted from the summary.
!      -- A summary of any erroneous fields encountered in the input data,
!         with each unique pattern and its frequency listed.
!
! (f) The test for NREC greater than or equal to 50 can be activated to stop
! before reading/printing the entire file.  Regarding feature (e), note that
! the {prnsum} summary will only be output if statement 900 is reached.
!
! (g) With more extensive modifications, {rdimma} can be adapted to write
! IMMA data, using non-IMMA data as input.  In this case it is important to
! note that a maintenance issue will arise as updates are made to {rdimma}.
! Therefore, we recommend that code added for this purpose be isolated (e.g.,
! into separate routines) as much as possible.  To adapt {rdimma}, the READ
! statement should be replaced by statements to read non-IMMA data, and
! {getrpt} deactivated.  Prior to reading the non-IMMA data, {init} should
! be called to properly initialize the data structures:
!     CALL INIT(CTRUE,ITRUE,FTRUE,FMISS,ILEN,NUM)
! Then the input data need to be transformed into the appropriate IMMA units,
! loaded into FTRUE (for numeric fields) or into CTRUE (for character fields),
! {putrpt} called, and the WRITE statement activated.  For writing character
! fields into IMMA, {minmax} (see feature (e)), must also be called to signal
! to the program when character data are present in CTRUE.  This also ensures
! that any data (numeric or character) not compliant with the format are not
! output (we also recommend that the other parts of feature (e), {savsum} and
! {prnsum}, be called).  As discussed in Data Representation note (2) below,
! base36 data (ITYPE=2) need to be stored in FTRUE as the corresponding numeric
! value (e.g., 9=9.0, A=10.0).  Data of ITYPE=3 (character) must be stored
! left-justified in CTRUE (see {bdimma} for field types and constraints).
!
! (h) Writing out new Unique IDs (UIDs): calls to {getuid} will return the UID
! number found in file LAST_UID on unit 98, incremented by one. Also, {getuid}
! then increments by one the UID number stored on unit 98. We emphasize that
! the ICOADS project manages the master list of UID assignments, thus new UIDs
! should not be assigned and written out without appropriate advance
! consultation.
!
! Data representation notes:
! (1) Printed data:  In the header, the field abbreviations (from ABBR) are
!     listed vertically.  The abbreviations are 1-5 characters in length, so
!     the header always occupies five lines (even if a selected subset of
!     fields includes no 5-character abbreviations).  All decimal points are
!     omitted for printing (i.e., each FTRUE value is divided by its FUNITS;
!     e.g., LON=33.80/0.01 = 3380 printed).  Missing data (FTRUE=FMISS) are
!     printed as blank, as are erroneous data, unless the default for FERR is
!     changed (see note 3).
! (2) Stored data (FTRUE, ITRUE, and CTRUE):  Some 1-character fields and ATTL
!     of attm8 (i.e., those with ITYPE=2) have alphabetic base36 values defined
!     (and printed) as part of the legal range.  In ITRUE and FTRUE, these are
!     stored as the corresponding numeric values: A=10, B=11, ..., Z=35.  For
!     character (ITYPE=3) fields, note that ITRUE and FTRUE contain the ICHAR
!     of the first character of the field (e.g., if ID=WHRN, ITRUE(ID)=87).
!     This simplifies the implementation of the program, but users should be
!     aware of the data type in processing character fields.
! (3) Erroneous data:  Erroneous data can be detected in two ways: (i) Data
!     that could not be read properly (e.g., a numeric field containing a
!     special character).  (ii) "Extreme" data, which read properly, but failed
!     to adhere to the ranges defined in {bdimma} (e.g., a numeric temperature
!     outside of -99.9 to 99.9).  Data of the first type are always set to
!     FERR, but data of the second type are only set to FERR if {minmax} is
!     called.  FERR equals FMISS by default.  If FERR is set to a non-default
!     value such as -888888., the printed output will contain all asterisks in
!     fields that contained erroneous data.  The non-default setting is also
!     used for quality control applications that need to distinguish between
!     actual missing data and data that were extant but erroneous.
!
! Performance:  The long character strings made necessary by the supplemental
! attm slow the program down.  Anyone not interested in those data could
! replace:
!      CHARACTER*1024 RPT,CTRUE
! with
!      CHARACTER*1024 RPT,CTRUE*16
! and
!      CALL GETRPT(RPT,CTRUE,ITRUE,FTRUE,FMISS,FERR                            &
!     &,ILEN,ABBR,FMIN1,FUNITS,ITYPE,RPTID,NUM)
! with
!      CALL GETRPT(RPT,CTRUE,ITRUE,FTRUE,FMISS,FERR                            &
!     &,ILEN,ABBR,FMIN1,FUNITS,ITYPE,RPTID,NUM-1)
!
! Machine/language dependencies:  None known.  Non-ANSI features: common block
! containing both character and non-character variables, and specification
! statements after data statements.
! The program conforms to Fortran 90 source code conventions for comments and
! continuation lines, but still compiles without change in many Fortran 77
! environments.  Added/included source code can be either free form or fixed.
! For more information: See
! <http://icoads.noaa.gov/e-doc/imma/R2.5-imma_short.pdf>
! (electronic document).
!-----------------------------------------------------------------------3456789
      PROGRAM RDIMMA
      IMPLICIT INTEGER(A-E,G-Z)
      INTEGER FNC,FM
!
      CHARACTER*(*) PROGID
      PARAMETER(PROGID='icoads-immt5Ximma1.01C')
!
      character*16 ccloud							!DEFINED FOR CLOUD TYPE AND CLOUD VARIABLES, 7/19/14
      character*1 ccldamt, ccldamt2, ccldamt3, ccldamt4				!CLOUD AMOUNTS FOR FIRST TWO CLOUD DECKS (F, S, B, O)
      character*4 clowcldamt							!WILL HOLD CLOUD AMOUNT FOR EACH OF 4 POTENTIAL LOW DECKS
      character*4 ccvconstant(8)							!CLOUD COVERAGE CONSTANTS
      integer icldh, icldh2, icldh3, icldh4					!CLOUD HEIGHTS FOR FIRST TWO CLOUD DECKS
      integer icldhm 								!CLOUD HEIGHT IN METERS (WILL CONVERT icldh to icldhm)
      integer icldcode								!CLOUD HEIGHT CONVERTED TO SYNOPTIC CODE
      integer ideckhgt								!SET TO EITHER 6500 FEET OR 20000 FEET
      integer itotlowcld, itotsky						!COMPUTED LOW CLOUD AMOUNT, REPORTED TOTAL CLOUD AMOUNT
      character*5 cvis								!VISIBILITY
      real fvis   								!VISIBILITY IN NAUTICAL MILES AS REPORTED
      real fvism								!VISIBILITY IN METERS (CONVERTED FROM NM)
      character wxel1(99)*5, wxel2(99)*5, wxel3(99)*5				!SYNOPTIC PRESENT WEATHER ELEMENTS (UP TO 3 POSSIBLE WX ELEMENTS PER)
      character cwx(3)*5							!EACH OF THE 3 METAR WEATHER ELEMENTS
      logical bwx(3)								!BOOLEAN FOR WHETHER OR NOT THERE IS A WEATHER ELEMENT
      character*3 cslp								!SLP (0.1hPa, last 3 digits)
      character*3 cwskt								!WIND SPEED (knots)
      integer islp, iwskt							!INTEGER VERSIONS OF SLP and WSKT
      character*3 cshipc							!SHIP COURSE, CHARACTER VARIABLE
      character*2 cships							!SHIP SPEED, CHARACTER VARIABLE
      integer ishipc, iships							!INTEGER, SHIP COURSE AND SPEED
      character*2 cwvht, cswht, cswht2						!WAVE HEIGHT AND SWELL HEIGHT (1 and 2), CHARACTER VARIABLE
      integer iwvht, iswht, iswht2						!INTEGER, WAVE AND SWELL HEIGHT (1 and 2)
      real fwvht_m, fswht_m, fswht2_m						!WAVE AND SWELL HEIGHT (1 and 2), IN METERS
      character*2 cwvprd							!INPUT WAVE OR SWELL PERIOD
      integer iwvprd								!INTEGER, WAVE OR SWELL PERIOD
      integer tp								!ASSIGNED VALUE FOR EITHER WP, SP, or SP2
      character*3 cwd			 					!WIND DIRECTION
      character*1 cquad								!QUADRANT OF WORLD THAT LAT/LON PERTAINS TO
!
      DATA (ccvconstant(i),i=1,8)/' ','F',' ','S',' ','B',' ','O'/
!
!ADD THE FOLLOWING FOR WEATHER ELEMENTS CONVERSION
      DATA wxel1/'xxxxx','xxxxx','xxxxx','   FU','xxxxx','   HZ',' BLSA'      & !1-7
     &,'   PO','   DS','   BR','xxxxx',' MIFG','xxxxx','Virga'                & !8-14
     &,'xxxxx',' VCSH','   TS','   SQ','xxxxx','   DZ','   RA'                & !15-21
     &,'   SN','   PE',' FZRA',' SHRA',' SHSN',' SHGR','   FG'                & !22-28
     &,'   TS','  -SS','   SS','xxxxx','xxxxx','  +SS','xxxxx'                & !29-35
     &,'-DRSN','+DRSN','-BLSN','+BLSN',' PRFG','xxxxx','xxxxx'                & !36-42
     &,'xxxxx','xxxxx','xxxxx','xxxxx','   FG','xxxxx',' FZFG'                & !43-49
     &,'xxxxx','  -DZ','xxxxx','   DZ','xxxxx','  +DZ','-FZDZ'                & !50-56
     &,' FZDZ','-RADZ',' RADZ','xxxxx','  -RA','xxxxx','   RA'                & !57-63
     &,'xxxxx','  +RA','-FZRA',' FZRA','-DZSN',' RASN','xxxxx'                & !64-70
     &,'  -SN','xxxxx','   SN','xxxxx','  +SN','   IC','  -SG'                & !71-77
     &,'xxxxx','  -PE','-SHRA',' SHRA','+SHRA','xxxxx','xxxxx'                & !78-84, 83&84 Blank. Handled by if statement in body of pgm.
     &,'-SHSN','+SHSN','-SHPE',' SHPE','xxxxx','   GR','xxxxx'                & !85-91
     &,'xxxxx','xxxxx','xxxxx',' TSRA',' TSGR','+TSSN',' TSSA'                & !92-98
     &,'+TSGR'/  							        !99
!
!IN SOME CASES THERE IS MORE THAN 1 OPTION FOR CONVERSION. THIS IS THE SECOND, WHERE APPLICABLE
      DATA wxel2/'xxxxx','xxxxx','xxxxx','xxxxx','xxxxx','xxxxx',' BLDU'      & !1-7
     &,'xxxxx','   SS','xxxxx','xxxxx','xxxxx','xxxxx','xxxxx'                & !8-14
     &,'xxxxx','-VCSH','xxxxx','xxxxx','xxxxx','xxxxx','xxxxx'                & !15-21
     &,'xxxxx','xxxxx','xxxxx','xxxxx','xxxxx','xxxxx',' FZFG'                & !22-28
     &,'   TS','  -DS','   DS','xxxxx','xxxxx','  +DS','xxxxx'                & !29-35
     &,' DRSN','xxxxx',' BLSN','xxxxx',' BCFG','xxxxx','xxxxx'                & !36-42
     &,'xxxxx','xxxxx','xxxxx','xxxxx','xxxxx','xxxxx','xxxxx'                & !43-49
     &,'xxxxx','xxxxx','xxxxx','xxxxx','xxxxx','xxxxx','xxxxx'                & !50-56
     &,'+FZDZ','xxxxx','+DZRA','xxxxx','xxxxx','xxxxx','xxxxx'                & !57-63
     &,'xxxxx','xxxxx','xxxxx','+FZRA','-RASN','+RASN','xxxxx'                & !64-70
     &,'xxxxx','xxxxx','xxxxx','xxxxx','xxxxx','xxxxx','   SG'                & !71-77
     &,'xxxxx','   PE','xxxxx','xxxxx','xxxxx','xxxxx','xxxxx'                & !78-84
     &,'xxxxx','xxxxx','xxxxx','+SHPE','xxxxx','+SHGR','xxxxx'                & !85-91
     &,'xxxxx','xxxxx','xxxxx',' TSSN','-TSPE','+TSRA','+TSSA'                & !92-98
     &,'+TSPE'/  							        !99
!
!IN SOME CASES THERE IS MORE THAN 1 OPTION FOR CONVERSION. THIS IS THE THIRD, WHERE APPLICABLE
      DATA wxel3/'xxxxx','xxxxx','xxxxx','xxxxx','xxxxx','xxxxx',' BLPY'      & !1-7
     &,'xxxxx','xxxxx','xxxxx','xxxxx','xxxxx','xxxxx','xxxxx'                & !8-14
     &,'xxxxx','xxxxx','xxxxx','xxxxx','xxxxx','xxxxx','xxxxx'                & !15-21
     &,'xxxxx','xxxxx','xxxxx','xxxxx','xxxxx','xxxxx','xxxxx'                & !22-28
     &,'xxxxx','xxxxx','xxxxx','xxxxx','xxxxx','xxxxx','xxxxx'                & !29-35
     &,'xxxxx','xxxxx','xxxxx','xxxxx','xxxxx','xxxxx','xxxxx'                & !36-42
     &,'xxxxx','xxxxx','xxxxx','xxxxx','xxxxx','xxxxx','xxxxx'                & !43-49
     &,'xxxxx','xxxxx','xxxxx','xxxxx','xxxxx','xxxxx','xxxxx'                & !50-56
     &,'xxxxx','xxxxx','xxxxx','xxxxx','xxxxx','xxxxx','xxxxx'                & !57-63
     &,'xxxxx','xxxxx','xxxxx','xxxxx','xxxxx','xxxxx','xxxxx'                & !64-70
     &,'xxxxx','xxxxx','xxxxx','xxxxx','xxxxx','xxxxx','  +SG'                & !71-77
     &,'xxxxx','  +PE','xxxxx','xxxxx','xxxxx','xxxxx','xxxxx'                & !78-84
     &,'xxxxx','xxxxx','xxxxx','xxxxx','xxxxx','xxxxx','xxxxx'                & !85-91
     &,'xxxxx','xxxxx','xxxxx','xxxxx','xxxxx','xxxxx','xxxxx'                & !92-98
     &,'xxxxx'/  							        !99
!
! CORE
      PARAMETER(YR=1,MO=2,DY=3,HR=4,LAT=5,LON=6,IM=7,ATTC=8,TI=9,LI=10        &
     &,DS=11,VS=12,NID=13,II=14,ID=15,C1=16,DI=17,D=18,WI=19,W=20,VI=21       &
     &,VV=22,WW=23,W1=24,SLP=25,A=26,PPP=27,IT=28,AT=29,WBTI=30,WBT=31        &
     &,DPTI=32,DPT=33,SI=34,SST=35,N=36,NH=37,CL=38,HI=39,H=40,CM=41          &
     &,CH=42,WD=43,WP=44,WH=45,SD=46,SP=47,SH=48)
!
! ICOADS ATTACHMENT
      PARAMETER(ATTI1=49,ATTL1=50,BSI=51,B10=52,B1=53,DCK=54,SID=55           &
     &,PT=56,DUPS=57,DUPC=58,TC=59,PB=60,WX=61,SX=62,C2=63,SQZ=64,SQA=65      &
     &,AQZ=66,AQA=67,UQZ=68,UQA=69,VQZ=70,VQA=71,PQZ=72,PQA=73,DQZ=74         &
     &,DQA=75,ND=76,SF=77,AF=78,UF=79,VF=80,PF=81,RF=82,ZNC=83,WNC=84         &
     &,BNC=85,XNC=86,YNC=87,PNC=88,ANC=89,GNC=90,DNC=91,SNC=92,CNC=93         &
     &,ENC=94,FNC=95,TNC=96,QCE=97,LZ=98,QCZ=99)
!
! IMMT-5/FM13 ATTACHMENT
      PARAMETER(ATTI5=100,ATTL5=101,OS=102,OP=103,FM=104,IMMV=105,IX=106      &
     &,W2=107,WMI=108,SD2=109,SP2=110,SH2=111,IS=112,ES=113,RS=114            &
     &,IC1=115,IC2=116,IC3=117,IC4=118,IC5=119,IR=120,RRR=121,TR=122          &
     &,NU=123,QCI=124,QI1=125,QI2=126,QI3=127,QI4=128,QI5=129,QI6=130         &
     &,QI7=131,QI8=132,QI9=133,QI10=134,QI11=135,QI12=136,QI13=137            &
     &,QI14=138,QI15=139,QI16=140,QI17=141,QI18=142,QI19=143,QI20=144         &
     &,QI21=145,HDG=146,COG=147,SOG=148,SLL=149,SLHH=150,RWD=151,RWS=152      &
     &,QI22=153,QI23=154,QI24=155,QI25=156,QI26=157,QI27=158,QI28=159         &
     &,QI29=160,RH=161,RHI=162,AWSI=163,IMONO=164)
!
! MODEL QUALITY CONTROL ATTACHMENT
      PARAMETER(ATTI6=165,ATTL6=166,CCCC=167,BUID=168,FBSRC=169,BMP=170       &
     &,BSWU=171,SWU=172,BSWV=173,SWV=174,BSAT=175,BSRH=176,SRH=177            &
     &,BSST=178,MST=179,MSH=180,BY=181,BM=182,BD=183,BH=184,BFL=185)
!
! SHIP METADATA ATTACHMENT
      PARAMETER(ATTI7=186,ATTL7=187,MDS=188,C1M=189,OPM=190,KOV=191           &
     &,COR=192,TOB=193,TOT=194,EOT=195,LOT=196,TOH=197,EOH=198,SIM=199        &
     &,LOV=200,DOS=201,HOP=202,HOT=203,HOB=204,HOA=205,SMF=206,SME=207        &
     &,SMV=208)
!
! NEAR-SURFACE OCEANOGRAPHIC DATA ATTACHMENT
      PARAMETER(ATTI8=209,ATTL8=210,OTV=211,OTZ=212,OSV=213,OSZ=214           &
     &,OOV=215,OOZ=216,OPV=217,OPZ=218,OSIV=219,OSIZ=220,ONV=221,ONZ=222      &
     &,OPHV=223,OPHZ=224,OCV=225,OCZ=226,OAV=227,OAZ=228,OPCV=229             &
     &,OPCZ=230,ODV=231,ODZ=232,PUID=233)
!
! UNIQUE ID ATTACHMENT
      PARAMETER(ATTI98=234,ATTL98=235,UID=236,RN1=237,RN2=238,RN3=239         &
     &,RSA=240,IRF=241)
!
! SUPPLEMENTAL DATA ATTACHMENT
      PARAMETER(ATTI99=242,ATTL99=243,ATTE=244,SUPD=245)
!
      PARAMETER(NUM=245)
      CHARACTER*8 ABBR,RPTID
      COMMON /IMMA1/ILEN(NUM),ABBR(NUM),FMIN1(NUM),FMAX1(NUM)                 &
     &,FMIN2(NUM),FMAX2(NUM),FUNITS(NUM),ITYPE(NUM),RPTID
!
      CHARACTER*1024 RPT,CTRUE*172
      DIMENSION CTRUE(NUM),ITRUE(NUM),FTRUE(NUM)
      DATA CTRUE/NUM*' '/,ITRUE/NUM*-999999/,FTRUE/NUM*-999999./              &
     &,FMISS/-999999./,FERR/-999999./,UNIT/10/,STDOUT/16/
!
! PRINT PROGRAM HEADER
!!      WRITE(STDOUT,*)PROGID
! DO NOT PRINT CORE HEADER OR CORE
!      CALL PRNSKP(YR,SH)
! DO NOT PRINT ATTACHMENT HEADER OR ATTACHMENT
!      CALL PRNSKP(ATTI1,ATTI5-1)
!      CALL PRNSKP(ATTI5,ATTI6-1) !! DO NOT WANT TO SKIP ATTACHMENT 5 SO COMMENT OUT THIS LINE
      CALL PRNSKP(ATTI6,ATTI7-1)
      CALL PRNSKP(ATTI7,ATTI8-1)
      CALL PRNSKP(ATTI8,ATTI98-1)
      CALL PRNSKP(ATTI98,ATTI99-1)
!      CALL PRNSKP(ATTI99,SUPD)
!
! PRINT REPORT HEADER
      CALL PRNHDR(STDOUT,ILEN,ABBR,YR,SUPD)
! PRINT CORE HEADER
!      CALL PRNHDR(STDOUT,ILEN,ABBR,YR,SH)
! PRINT ATTACHMENT HEADER
!      CALL PRNHDR(STDOUT,ILEN,ABBR,ATTI1,ATTI5-1)
!      CALL PRNHDR(STDOUT,ILEN,ABBR,ATTI5,ATTI6-1)
!      CALL PRNHDR(STDOUT,ILEN,ABBR,ATTI6,ATTI7-1)
!      CALL PRNHDR(STDOUT,ILEN,ABBR,ATTI7,ATTI8-1)
!      CALL PRNHDR(STDOUT,ILEN,ABBR,ATTI8,ATTI98-1)
!      CALL PRNHDR(STDOUT,ILEN,ABBR,ATTI98,ATTI99-1)
!      CALL PRNHDR(STDOUT,ILEN,ABBR,ATTI99,SUPD)
! INITIALIZE NUMBER OF REPORTS READ
      NREC=0
!
    iline=0

! READ REPORT
  100 CONTINUE
      iline=iline+1
      READ(*,'(A)',END=900)ctrue(supd)
    write(*,*) 'Read Line: ', iline
!
!-----------BEGINNING OF PRESENT WEATHER CONVERSIONS-------------------------
!
      cwx(1) = ctrue(supd)(50:54)
      cwx(2) = ctrue(supd)(55:59)
      cwx(3) = ctrue(supd)(60:64)
    
! DETERMINE IF A PRESENT WEATHER ELEMENT WAS REPORTED FOR ANY OF 3 POSSIBLE 
      do i=1,3
        if(cwx(i).eq.' ') then
	  bwx(i) = .false.
	else
	  bwx(i) = .true.
	endif
      enddo

! FOR EACH OBSERVED PRESENT WEATHER ELEMENT, DETERMINE IF THERE IS A MATCHING SYNOPTIC (IMMA) CODE
      do i=1,3 
        if(bwx(i)) then
	  do j=1,99
	    if(cwx(i).eq.wxel1(j).or.cwx(i).eq.wxel2(j).or.cwx(i).eq.wxel3(j)) then
!	      write(*,*) i,' immawx: ',j,'cwx: ',cwx(i)
	      write(ctrue(WW)(:16),'(i2.2)') j
	    endif
	  enddo
	endif
      enddo
! MAKE SPECIAL ACCOUNT OF ELEMENTS 83 AND 84, RAIN/SNOW MIXED
      do i=1,2 
        i2=i+1
        if(bwx(i).and.bwx(i2)) then
	  if((cwx(i).eq.'-SHRA'.and.cwx(i2).eq.'-SHSN').or.(cwx(i).eq.'-SHSN'.and.cwx(i2).eq.'-SHRA')) then
	    j=83
	    write(ctrue(WW)(:16),'(i2.2)') j
	  endif
	endif
        if(bwx(i).and.bwx(i2)) then
	  if((cwx(i).eq.' SHRA'.and.cwx(i2).eq.' SHSN').or.(cwx(i).eq.' SHSN'.and.cwx(i2).eq.' SHRA')) then
	    j=84
	    write(ctrue(WW)(:16),'(i2.2)') j
	  endif
	endif
      enddo



!------------END OF PRESENT WEATHER CONVERSIONS------------------------------
!
!-----------BEGINNING OF CLOUD TYPE/HEIGHT ASSIGNMENTS------------------------
! BEFORE WRITING ELEMENTS TO CTRUE, READ CLOUD AMOUNT AND CLOUD HEIGHTS FROM 
! THE CHARACTER STRING, DETERMINE LOWEST CLOUD HEIGHT, AND THEN WILL BE READY 
! TO WRITE TO THE PROPER CTRUE VARIABLE (7/19/14, jhl)
! NOTE THAT IN THE ABSENCE OF LOW CLOUDS (<6500 FEET) MID-LEVEL CLOUDS (<20,000 FEET) ARE INCLUDED
      ccloud = ctrue(supd)(65:80)
      if(ctrue(supd)(105:105).ne.' ') then			!IF TOTSKYCOVER IS REPORTED
	read(ctrue(supd)(105:105),'(i1)') itotsky
      else							!ELSE SET TO MISSING
	itotsky=-9999
      endif
!	
!IF SKY CONDITION IS CLEAR (SK or C) THEN LOWER CLOUD AMOUNT IS '0', and cloud height (H) is blank.
      if(ccloud(1:2).eq.'SK'.or.ccloud(1:1).eq.'C') then		
	 ctrue(NH)(:16)='0' 
         ctrue(H)(:16)=' '
!IF SKY CONDITION IS NOT CLEAR AND NOT MISSING (i.e., F,S,B,O), then read cloud heights for all potential layers
      else if(ccloud(1:2).ne.'SK'.and.ccloud(1:1).ne.'C'.and.ccloud(1:1).ne.' ') then
	ccldamt=ccloud(1:1)							!ASSIGN FIRST CLOUD AMOUNT
	if(ccloud(2:4).ne.' ') then
	  read(ccloud(2:4),'(i3)') icldh						!FIRST CLOUD HEIGHT
 	else
	  icldh = -9999
	endif
	ccldamt2=ccloud(5:5)							!ASSIGN SECOND CLOUD AMOUNT
	if(ccloud(6:8).ne.' ') then
	  read(ccloud(6:8),'(i3)') icldh2						!SECOND CLOUD HEIGHT
 	else
	  icldh2 = -9999
	endif
	ccldamt3=ccloud(9:9)							!ASSIGN THIRD CLOUD AMOUNT
	if(ccloud(10:12).ne.' ') then
	  read(ccloud(10:12),'(i3)') icldh3						!THIRD CLOUD HEIGHT
 	else
	  icldh3 = -9999
	endif
	ccldamt4=ccloud(13:13)							!ASSIGN FOURTH CLOUD AMOUNT
	if(ccloud(14:16).ne.' ') then
	  read(ccloud(14:16),'(i3)') icldh4						!FOURTH CLOUD HEIGHT
 	else
	  icldh4 = -9999
	endif
!
!IF THERE IS AT LEAST ONE DECK WITH CLOUDS AT OR BELOW 6500 FEET, THEN LOW CLOUDS ARE PRESENT
	if((icldh.gt.0.and.icldh.le.065).or.(icldh2.gt.0.and.icldh2.le.065).or.(icldh3.gt.0.and.icldh3.le.065).or.(icldh4.gt.0.and.icldh4.le.065)) then
	  ideckhgt=065
!ELSE LOOK FOR MID CLOUDS
	else if((icldh.gt.0.and.icldh.le.200).or.(icldh2.gt.0.and.icldh2.le.200).or.(icldh3.gt.0.and.icldh3.le.200).or.(icldh4.gt.0.and.icldh4.le.200)) then
	  ideckhgt=200
!ELSE CLOUD HEIGHTS NOT REPORTED (MISSING) -- SET DECK HEIGHT TO 0 SO NO FURTHER PROCESSING
!MANY TIMES A CLOUD AMOUNT IS REPORTED BUT NO CLOUD HEIGHT -- IN THOSE CASES CANNOT ASSUME CLOUDS ARE LOW
	else if(icldh.eq.-9999 .and. icldh2.eq.-9999 .and. icldh3.eq.-9999 .and. icldh4.eq.-9999) then
	  ideckhgt=-9999
!NO CLOUDS BELOW 20,000 FEET, SET IDECKHGT TO ZERO SO THAT NO FURTHER PROCESSING IS DONE BELOW
	else
	  ideckhgt=0
	endif
!
!IF CLOUDS ARE BELOW 6500 FEET OR BELOW 20000 FEET THEN PROCESS FURTHER
	if(ideckhgt.gt.0) then	
	 clowcldamt='    '			!INITIALIZE LOW CLOUD AMOUNTS TO MISSING
!FOR ALL CLOUD HEIGHTS AT OR BELOW 6500 FEET (OR 20,000 FEET), RETAIN THE CLOUD AMOUNT
	 if(icldh.le.ideckhgt) clowcldamt(1:1)=ccldamt
	 if(icldh2.le.ideckhgt) clowcldamt(2:2)=ccldamt2
	 if(icldh3.le.ideckhgt) clowcldamt(3:3)=ccldamt3
	 if(icldh4.le.ideckhgt) clowcldamt(4:4)=ccldamt4
!
!THIS CODE WAS INITIALLY INCORRECT -- SHOULD NOT SUM THE LAYER CLOUD AMOUNTS (SINCE EACH LAYER INCLUDES THE AMOUNT FROM THE LAYER(S) BELOW). RATHER, SIMPLY TAKE THE MAXIMUM CLOUD AMOUNT REPORTED FOR THE LOWER LAYERS (either 6500 FEET OR 20000 FEET)
! NOTE THAT BROKEN CLOUDS ARE SET TO A VALUE OF '6', ALTHOUGH THEY COULD BE SET TO '7'. FEW=2, SCT=4, OVC=8
	 itotlowcld=0
	 do i=1,4
	   do j=2,8,2
	     if(clowcldamt(i:i).eq.ccvconstant(j)) then		!FEW, SCT, BKN, OVC (2,4,6,8)
								!FEW (1/8to2/8); SCT (3/8to4/8); BKN (5/8-7/8); OVC (8/8)
	       if(j.gt.itotlowcld) itotlowcld=j
	     endif
	   enddo
	 enddo
!THERE SHOULD BE AT LEAST SOME CLOUDS OR ELSE SHOULDNT BE HERE
	 if(itotlowcld.lt.1) then
	   write(*,*) 'LOW CLOUD ERROR, STOP'
	   STOP
	 endif
!
!TOTAL LOW (OR MID) CLOUD AMOUNT CANNOT BE GREATER THAN METAR REPORTED TOTAL CLOUD AMOUNT
	 if(itotsky.ge.0) then							!ENSURE TOTAL CLOUD AMOUNT IS NOT MISSING (i.e., 0)
	   if(itotlowcld.gt.itotsky) itotlowcld=itotsky
	 endif
!
	 write(ctrue(NH),'(i1)') itotlowcld

!SET LOW CLOUD HEIGHT BY FINDING LOWEST HEIGHT AMONG ALL REPORTED (SOMETIMES NOT IN ORDER FROM LOWEST TO HIGHEST)
!IF SECOND, THIRD, OR FOURTH CLOUD DECK IS NOT MISSING AND THE LOWEST, THEN ASSIGN VALUE
	 if(ccldamt2.ne.' ') then
	   if(icldh2.lt.icldh.and.icldh2.lt.icldh3.and.icldh2.lt.icldh4) then						!SOMETIMES THE 2ND CLOUD DECK IS LOWEST
	     icldh=icldh2							!REASSIGN THE SECOND DECK CLOUD HEIGHT
	   endif
	 endif
	 if(ccldamt3.ne.' ') then
	   if(icldh3.lt.icldh.and.icldh3.lt.icldh2.and.icldh3.lt.icldh4) then						!SOMETIMES THE 3RD CLOUD DECK IS LOWEST
	     icldh=icldh3							
	   endif
	 endif
	 if(ccldamt4.ne.' ') then
	   if(icldh4.lt.icldh.and.icldh4.lt.icldh2.and.icldh4.lt.icldh3) then						!SOMETIMES THE 4TH CLOUD DECK IS LOWEST
	     icldh=icldh4							
	   endif
	 endif
!
! ASSIGN CLOUD HEIGHT (H) TO IMMA VARIABLE, BUT FIRST CONVERT FROM HUNDREDTHS OF FEET TO METERS
	 icldhm = NINT((icldh*100.0)*.3048)
! CONVERT METERS TO THE PROPER SYNOPTIC CODE (O TO 9)
! CLOUD CODES INCLUDED IN IMMA2.5 DOCUMENTATION
	 if(icldhm.le.50) then
	  icldcode=0
	 else if(icldhm.gt.50.and.icldhm.le.100) then
	  icldcode=1
	 else if(icldhm.gt.100.and.icldhm.le.200) then
	  icldcode=2
	 else if(icldhm.gt.200.and.icldhm.le.300) then
	  icldcode=3
	 else if(icldhm.gt.300.and.icldhm.le.600) then
	  icldcode=4
	 else if (icldhm.gt.600.and.icldhm.le.1000) then
	  icldcode=5
	 else if (icldhm.gt.1000.and.icldhm.le.1500) then
	  icldcode=6
	 else if (icldhm.gt.1500.and.icldhm.le.2000) then
	  icldcode=7
	 else if (icldhm.gt.2000.and.icldhm.le.2500) then
	  icldcode=8
	 else if (icldhm.gt.2500) then
	  icldcode=9
         endif
! AFTER CONVERSION WRITE CLOUD HEIGHT TO CLOUD HEIGHT VARIABLE (H)
!	 write(*,*) 'icldh, icldhm, icldcode: ', icldh, ' ',icldhm, icldcode
	 write(ctrue(H)(:16),'(i1)')icldcode
!
! THERE ARE NO CLOUDS AT OR BELOW 6500 FEET OR BELOW 20000 FEET, SO SKIES ARE CLEAR	
	else if(ideckhgt.eq.0) then
	 ctrue(NH)(:16)='0' 
         ctrue(H)(:16)=' '
! THERE WERE NO CLOUD HEIGHTS REPORTED SO CANT ASSUME CLOUDS WERE LOW OR MID, SO SET TO MISSING
	else
	 ctrue(NH)(:16)=' ' 
         ctrue(H)(:16)=' '
	endif
! THERE ARE NO CLOUD OBSERVATIONS (i.e., MISSING)
      else
	 ctrue(NH)(:16)=' ' 
         ctrue(H)(:16)='A'
      endif
!-----------END OF CLOUD TYPE/HEIGHT ASSIGNMENTS------------------------------
!
!-----------BEGINNING OF VISIBILITY ASSIGNMENTS-------------------------------
!
      cvis = ctrue(supd)(45:49)
!UNSURE IF VARIABLE VISIBILITY REPORTED. IF SO, STOP AND EVALUATE
      do i=1,5
	if(cvis(i:i).eq.'V') then
	  write(*,*) 'Variable visibility, STOP'
	  STOP
	endif
      enddo
!
      if(cvis.ne.' ') then
	read(cvis,'(f5)') fvis						!VISIBILITY FROM CHARACTER TO INTEGER
        fvism=fvis*1852.						!CONVERT NAUTICAL MILES TO METERS
!        write(*,*) 'fvis, fvism: ', fvis, ' ', fvism
      else
	fvism=FMISS							!IF VISIBILITY NOT REPORTED, SET TO MISSING
      endif

!VISIBILITY CODES IN IMMA2.5 DOCUMENTATION
      if (fvism.ne.fmiss) then
        if (nint(fvism).ge.50000) then
          ctrue(VV)(:16)='99'
        else if (nint(fvism).ge.20000) then
          ctrue(VV)(:16)='98'
        else if (nint(fvism).ge.10000) then
          ctrue(VV)(:16)='97'
        else if (nint(fvism).ge.4000) then
          ctrue(VV)(:16)='96'
        else if (nint(fvism).ge.2000) then
          ctrue(VV)(:16)='95'
        else if (nint(fvism).ge.1000) then
          ctrue(VV)(:16)='94'
        else if (nint(fvism).ge.500) then
          ctrue(VV)(:16)='93'
        else if (nint(fvism).ge.200) then
          ctrue(VV)(:16)='92'
        else if (nint(fvism).ge.50) then
          ctrue(VV)(:16)='91'
        else if (nint(fvism).ge.0) then
          ctrue(VV)(:16)='90'
        endif
      else
	ctrue(VV)(:16)='     '
      endif
!
!----------END OF VISIBILITY ASSIGNMENTS----------------------------------
!
!-----------BEGIN SEA LEVEL PRESSURE--------------------------------------
!
      IMISS = -999999

      cslp = ctrue(supd)(102:104)		!SLP, 0.1hPa, last 3 digits
      if(cslp.eq.' ') then
        islp = IMISS		
      else
	read(cslp,'(i3)') islp
      endif
     
      cwskt = ctrue(supd)(30:32)		!WIND SPEED IN KNOTS             (Use Wind Speed for Pressure Determination)
      if(cwskt.eq.' ') then
        iwskt = IMISS
      else
	read(cwskt,'(i3)') iwskt
      endif

      if(islp.ne.IMISS.and.iwskt.ne.IMISS) then		!SLP and WIND SPEED NOT MISSING
        if(iwskt.lt.64) then			!WINDS LESS THAN HURRICANE FORCE
	  if(islp.le.500) then			!Pressure less than 50.0. Assume values 00.0 to 50.0 are 1000.0 to 1050.0.
	    islp=islp+10000			!Add 10000 to SLP (0.1 hPa)
	  else					
	    islp=islp+9000		        !Assume values greater than 50.0 are 950 to 1000hPa. Add 900 to SLP after dividing by 10
	  endif
        else					!HURRICANE FORCE WINDS
          islp=islp+9000      	 		!Assume all values during hurricane force winds are in the 900 to 999.9 range (Assume Navy would not be traversing through sub-900 hPa areas.)
        endif
! WRITE SLP TO CTRUE
        write(ctrue(SLP)(:16),'(i5)') islp
      else					!SLP or WIND SPEED MISSING
! SET SLP TO MISSING IN CTRUE			!SLP SET TO MISSING IF WIND SPEED IS NOT REPORTED
        ctrue(SLP)(:16) = ' '
      endif
!
!------------END SEA LEVEL PRESSURE---------------------------------------
!
!------------BEGINNING OF SHIP COURSE AND SHIP SPEED CONVERSION--------
      cshipc = ctrue(supd)(106:108)		!SHIP COURSE
      cships = ctrue(supd)(109:110)		!SHIP SPEED

      if(cshipc.ne.' ') then			!SHIP COURSE REPORTED
        read(cshipc,'(i3)') ishipc		!CONVERT SHIP COURSE TO INTEGER IF NOT BLANK
      else					!IF SHIP COURSE BLANK, THEN SET IMMA CODE TO MISSING ('9')
	ctrue(DS)(:16)='9'			!SHIP COURSE UNKNOWN
	ishipc=-99
      endif
!   write(*,*) 'Ship course: ', cshipc, ' ',ishipc
      if(cships.ne.' ') then			!SHIP SPEED REPORTED
        read(cships,'(i2)') iships		!CONVERT SHIP SPEED TO INTEGER IF NOT BLANK
      else					!SHIP SPEED MISSING
	ctrue(VS)(:16)=' '			!IMMA CODE - SHIP SPEED SET TO MISSING (BLANK)
	iships=-99
      endif
!   write(*,*) 'Ship speed: ', cships, ' ',iships

! ASSIGN SHIP COURSE CODE (WMO CODE 0700 for true direction of resultant displacement of the ship the 3 hours preceding the time of observation)
      if(iships.eq.0) ctrue(DS)(:16)='0'				!SHIP STATIONARY (THIS ONLY RELEVANT IF SHIP COURSE MISSING AND SHIP SPEED REPORTED AS ZERO - THE DATASET CONTAINS SHIP SPEED OF ZERO AND SHIP COURSE)
      if(ishipc.gt.22.and.ishipc.le.67) ctrue(DS)(:16)='1'		!SHIP MOVING NE
      if(ishipc.gt.67.and.ishipc.le.112) ctrue(DS)(:16)='2'		!SHIP MOVING E
      if(ishipc.gt.112.and.ishipc.le.157) ctrue(DS)(:16)='3'		!SHIP MOVING SE
      if(ishipc.gt.157.and.ishipc.le.202) ctrue(DS)(:16)='4'		!SHIP MOVING S
      if(ishipc.gt.202.and.ishipc.le.247) ctrue(DS)(:16)='5'		!SHIP MOVING SW
      if(ishipc.gt.247.and.ishipc.le.292) ctrue(DS)(:16)='6'		!SHIP MOVING W
      if(ishipc.gt.292.and.ishipc.le.337) ctrue(DS)(:16)='7'		!SHIP MOVING NW
      if(ishipc.gt.337.or.(ishipc.ge.0.and.ishipc.le.22)) ctrue(DS)(:16)='8'	!SHIP MOVING N

! ASSIGN SHIP SPEED CODE (WMO CODE 4451 for ships average speed made good during the 3 hours preceding the time of observation)
      if(iships.eq.0) ctrue(VS)(:16)='0'
      if(iships.ge.1.and.iships.le.5) ctrue(VS)(:16)='1'
      if(iships.ge.6.and.iships.le.10) ctrue(VS)(:16)='2'
      if(iships.ge.11.and.iships.le.15) ctrue(VS)(:16)='3'
      if(iships.ge.16.and.iships.le.20) ctrue(VS)(:16)='4'
      if(iships.ge.21.and.iships.le.25) ctrue(VS)(:16)='5'
      if(iships.ge.26.and.iships.le.30) ctrue(VS)(:16)='6'
      if(iships.ge.31.and.iships.le.35) ctrue(VS)(:16)='7'
      if(iships.ge.36.and.iships.le.40) ctrue(VS)(:16)='8'
      if(iships.gt.40) ctrue(VS)(:16)='9'


!----------------END OF SHIP COURSE AND SHIP SPEED CONVERSION----------
!
!--------------BEGINNING OF WAVE HEIGHT AND SWELL HEIGHT CONVERSIONS---
      cwvht = ctrue(supd)(117:118)
      if(cwvht.ne.' ') then
	read(cwvht,'(i2)') iwvht
      else
	iwvht=-99
      endif
      if(iwvht.ne.-99) then
	fwvht_m = iwvht*0.3048					!CONVERT WAVE HEIGHT IN FEET TO METERS
!    write(*,*) 'Wave Height: ', iwvht, ' ', fwvht_m
        write(ctrue(WH)(:16),'(i2)') nint(fwvht_m*2.)		!CONVERT WAVE HEIGHT TO CODE (MULTIPLY METERS BY 2) AND WRITE TO IMMA ELEMENT
!    write(*,*) 'Wave Height Code: ', ctrue(WH)(:16)
      else
	ctrue(WH)(:16) = ' '
      endif

      cswht = ctrue(supd)(123:124)
      if(cswht.ne.' ') then
	read(cswht,'(i2)') iswht
      else
	iswht=-99
      endif
      if(iswht.ne.-99) then
        fswht_m = iswht*0.3048					!CONVERT SWELL HEIGHT IN FEET TO METERS
        write(ctrue(SH)(:16),'(i2)') nint(fswht_m*2.)		!CONVERT SWELL HEIGHT TO CODE (MULTIPLY METERS BY 2) AND WRITE TO IMMA ELEMENT
      else
	ctrue(SH)(:16) = ' '
      endif

      cswht2 = ctrue(supd)(129:130)
      if(cswht2.ne.' ') then
	read(cswht2,'(i2)') iswht2
      else
	iswht2=-99
      endif
      if(iswht2.ne.-99) then
        fswht2_m = iswht2*0.3048				!CONVERT SWELL HEIGHT IN FEET TO METERS
        write(ctrue(SH2)(:16),'(i2)') nint(fswht2_m*2.)		!CONVERT SWELL HEIGHT TO CODE (MULTIPLY METERS BY 2) AND WRITE TO IMMA ELEMENT
      else
	ctrue(SH2)(:16) = ' '
      endif
!
!----------------END OF WAVE HEIGHT AND SWELL HEIGHT CONVERSIONS-------
!
!_______________BEGINNING OF WAVE AND SWELL PERIOD CONVERSIONS---------

! THIS WAS INITIALLY CODED FOR A CONVERSION TO TABLE D5 (INCORRECTLY) FOR WAVE PERIOD. NO WAVE PERIOD CONVERSION IS NEEDED POST 1968.
!1=WAVE PERIOD, 2=SWELL PERIOD ONE, 3=SWELL PERIOD TWO

      ctrue(WP)(:16) = ctrue(supd)(115:116)
      ctrue(SP)(:16) = ctrue(supd)(121:122)
      ctrue(SP2)(:16) = ctrue(supd)(127:128)

!   write(*,*) 'WAVE PERIOD: ', ctrue(WP)(:16)
!   write(*,*) 'SWELL PERIOD 1: ', ctrue(SP)(:16)
!   write(*,*) 'SWELL PERIOD 2: ', ctrue(SP2)(:16)


!__________________END OF WAVE AND SWELL PERIOD CONVERSIONS------------
!
!__________________BEGINNING OF SWELL DIRECTION (NO WAVE DIRECTION)-----

! NO CONVERSION NECESSARY FOR SWELL DIRECTION ONE OR TWO
!
      ctrue(SD)(:16) = ctrue(supd)(119:120)			!SWELL DIRECTION ONE
      ctrue(SD2)(:16) = ctrue(supd)(125:126)			!SWELL DIRECTION TWO
!    write(*,*) 'SWELL DIRECTION: ', ctrue(SD)(:16)
!
!______________________END OF SWELL DIRECTION (NO WAVE DIRECTION)-------
!
!__________________BEGINNING OF WIND DIRECTION ADJUSTMENTS---------------
!
! cwskt is assigned in the SLP section above.
! ALSO note that there are ~400 reports of wind direction=362 in the Navy METAR records (~0.1% of observations).
! According to Eric Freeman variable winds were keyed as 362. This is equal to the indicator in IMMA for variable winds so no conversion is necessary.

      cwd=ctrue(supd)(26:28)

      if(cwd.eq.'  0'.or.cwd.eq.' 00') cwd='360'	!SET WIND DIRECTION TO 360 IF WRITTEN IN METAR AS ZERO
							!WINDS SHOULD BE CALM (00) IF DIRECTION IS ZERO BUT IS OFTEN NOT THE CASE
      if(cwskt.eq.'  0') cwd='361'			!SET WIND DIRECTION TO 'CALM' INDICATOR (361) IF WIND SPEED IS ZERO

      ctrue(D)(:16)=cwd

!_________________________END OF WIND DIRECTION ADJUSTMENTS---------------
!
!!__________________BEGINNING OF READING LAT/LON AND DETERMING QUADRANT----------------
!
      cquad=ctrue(supd)(13:13)
      ctrue(LAT)(:16)=' '//ctrue(supd)(14:16)//'0'		!LAT,	ADDING BLANK AT BEGINNING
								!ALSO ADDING '0' AT END FOR HUNDREDTHS
      ctrue(LON)(:16)=' '//ctrue(supd)(17:20)//'0'		!LON

      if(cquad.eq.'3'.or.cquad.eq.'5') then		!SOUTHERN HEMISPHERE
        ctrue(LAT)(1:1)='-'
      endif
      if(cquad.eq.'5'.or.cquad.eq.'7') then		!WESTERN HEMISPHERE
	ctrue(LON)(1:1)='-'
      endif
!
!_________________________END OF READING LAT/LON AND DETERMINING QUADRANT---------------
!     
!
! INCREMENT NUMBER OF REPORTS READ
! THE COLUMN LOCATIONS FOR EACH VARIABLE ARE DEFINED IN THE NAVY METAR TRANSPEC
      NREC=NREC+1
      ctrue(YR)(:16)=ctrue(supd)(1:4)
      ctrue(MO)(:16)=ctrue(supd)(5:6)
      ctrue(DY)(:16)=ctrue(supd)(7:8)
      ctrue(HR)(:16)=ctrue(supd)(9:12)
      ctrue(ID)(:16)=ctrue(supd)(21:24)			!CALL SIGN
      ctrue(W)(:16)=ctrue(supd)(30:32)			!WIND SPEED, 		CONVERT FROM KNOTS TO 0.1 m/s	READ 3 DIGITS INSTEAD OF 4 FOR CONSISTENCY WITH IMMA LENGTH. UNLIKELY TO BE MORE THAN 999 KNOT WIND SPEED. :)
      ctrue(AT)(:16)=ctrue(supd)(81:84)			!AIR TEMPERATURE
      ctrue(DPT)(:16)=ctrue(supd)(85:88)		!DEW POINT TEMPERATURE
      ctrue(WBT)(:16)=ctrue(supd)(89:92)		!WET BULB TEMPERATURE	
      ctrue(N)(:16)=ctrue(supd)(105:105)		!TOTAL CLOUD AMOUNT
      ctrue(SST)(:16)=ctrue(supd)(111:114)		!SST
!!! . JHLNAVY ADD DONE
!!! .
!!! .
!!! .
      ctrue(im)(:16)=' 1'				!IMMA VERSION
      ctrue(atti1)(:16)=' 1'				!ATTACHMENT 1 ID
      ctrue(attl1)(:16)='65'				!ATTACHMENT 1 LENGTH
      ctrue(atti5)(:16)=' 5'				!ATTACHMENT 5 ID
      ctrue(attl5)(:16)='94'				!ATTACHMENT 5 LENGTH
      ctrue(atti99)(:16)='99'				!ATTACHMENT 99 ID????
      ctrue(attl99)(:16)=' 0'				!ATTACHMENT 99 LENGTH

! The following is writing the elements and the original report to the variable 'rpt', which is subsequently converted from characters to floating point values in 'CALL GETRPT'.
      write(*,110)(ctrue(i),i=1,atti6-1),(ctrue(i),i=atti99,supd)
      write(rpt,110)(ctrue(i),i=1,atti6-1),(ctrue(i),i=atti99,supd)		!atti99,supd -- ORIGINAL REPORT AT THE END IN THE SUPPLEMENTAL ATTACHMENT


  110 format(a4,2a2,a4,a5,a6,a2,5a1,2a2,a9,a2,a1,a3,a1,a3,a1,2a2,a1,a5 &	!25 (1-48, Table C0 Core)
      ,a1,a3,a1,a4,a1,a4,a1,a4,a2,a4,7a1,8a2,a1,a3,a2,2a3,2a2,5a1,a2 &		!63 (49-99, Table C1 ICOADS attm) Last 2 in 8a2 are in Table C1. Goes through No. 15 in Table C1.
      ,33a1,a2,a1,a2 &								!Split a2 and moved to next line - which is the start of Table C5 (IMMT-5/FM-13)
      ,2a2,7a1,3a2,a1,a2,7a1,a3,24a1,2a3,2a2,3a3,8a1,a4,2a1 &			!Table C5, IMMA-Rev-v16.pdf
      ,a7 &									!End of Table C5, 164
      ,2a2,a1,a)								!242-245 (Table C99, ATTI=99, Supplemental Data Attachment)
!
! CONVERT CHARACTERS TO FLOATING POINT VALUES
      CALL GETRPT(RPT,CTRUE,ITRUE,FTRUE,FMISS,FERR                            &
     &,ILEN,ABBR,FMIN1,FUNITS,ITYPE,RPTID,NUM)
      ftrue(dck)=708.
      ftrue(sid)=109.
!
! CONVERT LONGITUDE TO DEGREES EAST
      CALL EAST(ITRUE(LON),FTRUE(LON),FERR)
	write(*,*) 'EastI: ', itrue(lon),' EastF: ',ftrue(lon)
! GET UNIQUE REPORT ID
!      CALL GETUID(CTRUE(UID))
! Add the following for conversion of wind speed, JHL, 8/26/14
! CONVERT WINDS FROM KNOTS TO M/S
!   write(*,*) 'WIND SPEED: ', ctrue(W), ' ', itrue(w)
      CALL FXKTMS(FTRUE(W))
! SET EXTREME FLOATING POINT VALUES TO ERROR VALUE
      CALL MINMAX(CTRUE,ITRUE,FTRUE,FMISS,FERR                                &
     &,ILEN,FMIN1,FMAX1,FMIN2,FMAX2,FUNITS,ITYPE,NUM)

!------------------------------------------------------------------------------------------
!----------------------SET ALL INDICATORS--------------------------------------------------
!------------------------------------------------------------------------------------------
!SET TIME INDICATOR - ALL TIMES ARE HOUR PLUS MINUTES (TI=2)
         if (ftrue(hr).ne.fmiss) ftrue(ti)=2.
!SET LAT/LON INDICATOR - ALL LOCATIONS ARE DEGREES AND TENTHS (LI=0)
         if (ftrue(lat).ne.fmiss .or. ftrue(lon).ne.fmiss) ftrue(li)=0.
!-----------------------SET WIND DIRECTION INDICATOR - 360-POINT COMPASS (DI=5)------------
         if(ftrue(D).ne.fmiss) then				!CONFIRM WIND DIRECTION IS NON-MISSING BEFORE SETTING DI
           ftrue(DI)=5
         else
           ftrue(DI)=fmiss				!ENSURE INDICATOR IS MISSING IF WIND DIRECTION IS MISSING
         endif
!--------------------SET WIND SPEED INDICATOR - KNOTS, MEASURED (WI=4)----------------
         if (ftrue(w).ne.fmiss .or. ftrue(rws).ne.fmiss) ftrue(wi)=4.
!--------------SET VISIBILITY INDICATOR TO '0' (ESTIMATED/UNKNOWN)
         if(ftrue(VV).ne.fmiss) then
 	   ftrue(VI)=0
         else
 	   ftrue(VI)=fmiss
         endif
!--------------SET INDICATORS OF (AIR) TEMPERATURE - ALL OBS ARE IN TENTHS DEG C (IT=0)
         if (ftrue(at).ne.fmiss .or. ftrue(wbt).ne.fmiss .or. &
          ftrue(dpt).ne.fmiss .or. ftrue(sst).ne.fmiss) ftrue(it)=0.
!-----------SET INDICATORS OF WET BULB TEMPERATURE - ASSUME ALL WERE MEASURED (WBTI=0)
         if (ftrue(wbt).ne.fmiss) ftrue(wbti)=0.
!-----------SET INDICATORS OF DEW POINT TEMPERATURE - ASSUME ALL WERE COMPUTED (DPTI=1)
         if (ftrue(dpt).ne.fmiss) ftrue(dpti)=1.
!-----------SET INDICATORS OF SST METHOD INDICATOR - ALL OBS ARE UKNOWN/NON-BUCKET (SI= 9)
!SST METHOD IS NOT REPORTED IN NAVY METAR
         if (ftrue(sst).ne.fmiss) ftrue(si)=9.
!-----------SET CLOUD HEIGHT INDICATOR------------------------------------------------
! CLOUD HEIGHT INDICATOR IS ALWAYS SET TO ESTIMATED FOR NAVY METAR - '0'
	 if (ftrue(h).ne.fmiss) then
	   ftrue(hi)=0.
	 else
	   ftrue(hi)=fmiss
	 endif
!---------SET INDICATORS HERE FOR THOSE WHERE WE DONT NEED TO CONFIRM NON-MISSING, ASSUME ALWAYS PRESENT----
!---------SET ID INDICATOR - ALL IDS ARE SHIP (II= 1)
         ftrue(II)=1.
!---------SET PLATFORM TYPE - US NAVY (PT= 0)
         ftrue(PT)=0.
!---------SET WAVE MEASUREMENT INDICATOR, SET TO 0, ESTIMATED-----------------------------
! On 1/6/15 conference call, Scott recommended WMI be set to missing in all cases since wave recorders not likely in NAVY METAR. 
! Setting WMI=missing is the appropriate choice, because there isnt any input field on which to base this setting.
! Note from the description of WMI in the IMMA documentation http://icoads.noaa.gov/e-doc/imma/R2.5-imma_short.pdf that WMI=0 means
	 ftrue(wmi)=fmiss
!---------COUNTRY CODE INDICATOR, SET TO 2, FOR US----------------------------------------
	 ftrue(C1)=2.
!-------NO RELATIVE HUMIDITY MEASUREMENTS---------
!!!      if (ftrue(rh).ne.fmiss) ftrue(rhi)=
!_________________________END OF SET INDICATOR CONSTANTS-------------------
!------------------------------------------------------------------------------------------
!------------------------------------------------------------------------------------------
!
! SAVE SUMMARY INFORMATION -- SAVING INFO SO IT CAN MAKE FORT.10 REPORT
      CALL SAVSUM(CTRUE,FTRUE,FMISS,FERR                                      &
     &,ILEN,ABBR,NUM)
!
! CONVERT FLOATING POINT VALUES TO CHARACTERS
      CALL PUTRPT(RPT,CTRUE,ITRUE,FTRUE,FMISS,FERR                            &
     &,ILEN,ABBR,FMIN1,FUNITS,ITYPE,RPTID,NUM)
! WRITE REPORT
!??i do not see a variable TRIM defined anywhere and dont see output to standard output when program runs?? -- FORT.16 IS STDOUT
! TRIM IS FORTRAN AND TAKES OFF ALL THE TRAILING BLANKS
      WRITE(STDOUT,'(A)')TRIM(RPT)
!
!??why are these print report statements commented out, particularly the attachments?? BECAUSE USING WRITE STATEMENTABOVBE - ONE YOU KNOW AND WANT TO MAKE FIXED LENGTH - KNOW WAHTS GOING ON - CAN DO ANOTHER OPTIOJN.
! PRINT REPORT
!!      CALL PRNRPT(STDOUT,ILEN,CTRUE,YR,SUPD)
! PRINT CORE
!      CALL PRNRPT(STDOUT,ILEN,CTRUE,YR,SH)
! PRINT ATTACHMENT
!      CALL PRNRPT(STDOUT,ILEN,CTRUE,ATTI1,ATTI5-1)
!      CALL PRNRPT(STDOUT,ILEN,CTRUE,ATTI5,ATTI6-1)
!      CALL PRNRPT(STDOUT,ILEN,CTRUE,ATTI6,ATTI7-1)
!      CALL PRNRPT(STDOUT,ILEN,CTRUE,ATTI7,ATTI8-1)
!      CALL PRNRPT(STDOUT,ILEN,CTRUE,ATTI8,ATTI98-1)
!      CALL PRNRPT(STDOUT,ILEN,CTRUE,ATTI98,ATTI99-1)
!      CALL PRNRPT(STDOUT,ILEN,CTRUE,ATTI99,SUPD)
!
! STOP AFTER SEVERAL REPORTS HAVE BEEN READ
!      IF (NREC.GE.50) STOP 'REMOVE STOP TO READ ALL REPORTS'
      call INIT(CTRUE,ITRUE,FTRUE,FMISS,ILEN,NUM)
      GOTO 100
!
! END OF FILE
  900 CONTINUE
!!      WRITE(STDOUT,*)'REPORTS ',NREC
!
! PRINT SUMMARY INFORMATION TO UNIT -- FORT.10 COMING FROM HERE
      CALL PRNSUM(UNIT,PROGID,ABBR,NUM)
      END
!=============================================================================!
! WARNING:  Code beyond this point should not require any modification.       !
!=============================================================================!
!-----------------------------------------------------------------------3456789
      SUBROUTINE INIT(CTRUE,ITRUE,FTRUE,FMISS,ILEN,NUM)
! INITIALIZE CTRUE, ITRUE, AND FTRUE
      IMPLICIT INTEGER(A-E,G-Z)
      CHARACTER*(*) CTRUE
      DIMENSION CTRUE(NUM),ITRUE(NUM),FTRUE(NUM),ILEN(NUM)
!
      DO 190 I=1,NUM
        IF (ILEN(I).EQ.1024) THEN
          CTRUE(I)(:)=' '
        ELSE
          IF (ILEN(I).GT.LEN(CTRUE(I))) STOP 'INIT LEN CTRUE'
          CTRUE(I)(:ILEN(I))=' '
        ENDIF
        ITRUE(I)=NINT(FMISS)
        FTRUE(I)=FMISS
  190 CONTINUE
      END
!-----------------------------------------------------------------------3456789
      SUBROUTINE PRNSKP(BEG,END)
! DO NOT PRINT REPORT HEADER OR REPORT
      IMPLICIT INTEGER(A-E,G-Z)
      CHARACTER*(*) ABBR,CTRUE
      DIMENSION ILEN(*),ABBR(*),CTRUE(*)
      CHARACTER*1024 HDR,RPT
      EQUIVALENCE(HDR,RPT)
      DIMENSION SKP(1024)
      DATA SKP/1024*0/
      SAVE SKP
!
      DO 190 I=BEG,END
        SKP(I)=1
  190 CONTINUE
      RETURN
!-----------------------------------------------------------------------3456789
      ENTRY PRNHDR(STDOUT,ILEN,ABBR,BEG,END)
! PRINT REPORT HEADER
!
      DO 290 J=1,5
        HDR=' '
        PTR=0
        DO 280 I=BEG,END
          IF (SKP(I).NE.1) THEN
            IF (ILEN(I).EQ.1024) THEN
              PTR=PTR+1
            ELSE
              PTR=PTR+ILEN(I)
            ENDIF
            IF (PTR.GT.LEN(HDR)) STOP 'PRNHDR LEN HDR'
            HDR(PTR:PTR)=ABBR(I)(J:J)
          ENDIF
  280   CONTINUE
        WRITE(STDOUT,'(A)')HDR(:PTR)
  290 CONTINUE
      RETURN
!-----------------------------------------------------------------------3456789
      ENTRY PRNRPT(STDOUT,ILEN,CTRUE,BEG,END)
! PRINT REPORT
!
      RPT=' '
      PTR=0
      DO 390 I=BEG,END
        IF (SKP(I).NE.1) THEN
          IF (ILEN(I).EQ.1024) THEN
            IF (CTRUE(I)(:).EQ.' ') THEN
              WRITE(STDOUT,'(2A)')RPT(:PTR),' '
            ELSE
              WRITE(STDOUT,'(2A)')RPT(:PTR),TRIM(CTRUE(I))
            ENDIF
            RETURN
          ENDIF
          IF (ILEN(I).GT.LEN(CTRUE(I))) STOP 'PRNRPT LEN CTRUE'
          IF (PTR+ILEN(I).GT.LEN(RPT)) STOP 'PRNRPT LEN RPT'
          RPT(PTR+1:PTR+ILEN(I))=CTRUE(I)(:ILEN(I))
          PTR=PTR+ILEN(I)
        ENDIF
  390 CONTINUE
      WRITE(STDOUT,'(A)')RPT(:PTR)
      END
!-----------------------------------------------------------------------3456789
      SUBROUTINE GETRPT(RPT,CTRUE,ITRUE,FTRUE,FMISS,FERR                      &
     &,ILEN,ABBR,FMIN1,FUNITS,ITYPE,RPTID,NUM)
! CONVERT CHARACTERS TO FLOATING POINT VALUES
      IMPLICIT INTEGER(A-E,G-Z)
      CHARACTER*(*) RPT,CTRUE,ABBR,RPTID
      DIMENSION CTRUE(NUM),ITRUE(NUM),FTRUE(NUM)                              &
     &,ILEN(NUM),ABBR(NUM),FMIN1(NUM),FUNITS(NUM),ITYPE(NUM)
      CHARACTER B36*5,IFMT(9)*8,STR*1024
      DATA IFMT/'(BZ,I1)','(BZ,I2)','(BZ,I3)','(BZ,I4)'                       &
     &,'(BZ,I5)','(BZ,I6)','(BZ,I7)','(BZ,I8)','(BZ,I9)'/
      SAVE IFMT
!
      PTR=0
      ATTC=0
      ATTI=0
      TRS=0
      DO 180 I=1,NUM
!	write(*,*) 'I: ', i,' PTR: ', ptr
!	write(*,*) '   ', i,' TRS: ', trs
!	write(*,*) 'str(trs+1:trs+4): ', STR(TRS+1:TRS+4)
!	write(*,*) 'ATTI: ', atti
        IF (ABBR(I).EQ.' ATTI') THEN						!Enter here when new attachment starts
          ATTI=NINT(FMIN1(I))							!Assign ATTI the FMIN1 value (is the attachment number)
          ATTL=NINT(FMIN1(I+1))							!Assign ATTL the FMIN1 value (is the attachment length)
          IF (ATTL.GT.99) THEN							!Assign the first 4 characters of STR as ATTI and ATTL
            WRITE(STR(TRS+1:TRS+4),'(I2,A2)')ATTI,B36(ATTL,'**',2)
          ELSE
            WRITE(STR(TRS+1:TRS+4),'(2I2)')ATTI,ATTL
          ENDIF
          IF (ATTI.EQ.99) THEN							!Enter here if the last attachment, 99
            IF (TRS+5.GT.LEN(STR)) STOP 'GETRPT LEN STR'
            IF (PTR+6.GT.LEN(RPT)) STOP 'GETRPT LEN RPT'
            IF (STR(TRS+1:TRS+4).NE.RPT(PTR+1:PTR+4)) THEN			!Enter here if first 4 characters of STR don't equal first 4 characters of RPT
!	write(*,*) 'str(trs+1:trs+4): ', STR(TRS+1:TRS+4)
!	write(*,*) '----'
!	write(*,*) 'rpt(ptr+1:ptr+4): ', RPT(PTR+1:PTR+4)
!	write(*,*) '----'
!	write(*,*) 'rpt(ptr+1): ', RPT(PTR+1:)
              IF (RPT(PTR+1:).NE.' ') STOP 'GETRPT ATTI ATTL'			!STOP if first 2 characters of RPT are not blank
              STR(TRS+1:TRS+5)=' '
            ELSE								!Else enter here if first 4 characters of STR EQUAL first 4 characters of RPT
              ATTC=ATTC+1
              STR(TRS+1:TRS+5)=RPT(PTR+1:PTR+5)
              PTR=PTR+5
            ENDIF
            IF (NUM.EQ.I+3) THEN
              CTRUE(NUM)(:)=RPT(PTR+1:)
              IF (CTRUE(NUM)(:).NE.RPT(PTR+1:)) STOP 'GETRPT LEN SUPD'
            ENDIF
          ELSE									!Entering here if this is start of next attachment and it's not the last attachment 99
            IF (TRS+ATTL.GT.LEN(STR)) STOP 'GETRPT LEN STR'
            IF (PTR+ATTL.GT.LEN(RPT)) STOP 'GETRPT LEN RPT'
            IF (STR(TRS+1:TRS+4).NE.RPT(PTR+1:PTR+4)) THEN
              STR(TRS+1:TRS+ATTL)=' '
            ELSE
              ATTC=ATTC+1							!Not sure what ATTC is??
              STR(TRS+1:TRS+ATTL)=RPT(PTR+1:PTR+ATTL)
              PTR=PTR+ATTL							!Increment PTR the full length of the new attachment
            ENDIF
          ENDIF
        ELSE IF (ATTI.EQ.0) THEN						!Enter here when Attachment=0, CORE
          IF (TRS+ILEN(I).GT.LEN(STR)) STOP 'GETRPT LEN STR'
          IF (PTR+ILEN(I).GT.LEN(RPT)) STOP 'GETRPT LEN RPT'
          STR(TRS+1:TRS+ILEN(I))=RPT(PTR+1:PTR+ILEN(I))
          PTR=PTR+ILEN(I)							!For some reason PTR gets incremented the length of the variable every loop when attachment=0, CORE
          IF (ABBR(I).EQ.'   IM') THEN
          ENDIF
        ENDIF
        TRS=TRS+ILEN(I)
  180 CONTINUE
!
      TRS=0
      DO 190 I=1,NUM
!     write(*,*) 'In GETRPT I: ', I
        IF (ILEN(I).NE.1024) THEN
          IF (ILEN(I).GT.LEN(CTRUE(I))) STOP 'GETRPT LEN CTRUE'
          CTRUE(I)(:ILEN(I))=STR(TRS+1:TRS+ILEN(I))
          TRS=TRS+ILEN(I)
        ENDIF
!
        IF (CTRUE(I)(:MIN(ILEN(I),LEN(CTRUE(I)))).EQ.' ') THEN
          ITRUE(I)=NINT(FMISS)
        ELSE IF (ITYPE(I).EQ.1) THEN
          READ(CTRUE(I)(:ILEN(I)),IFMT(ILEN(I)),IOSTAT=IOS)ITRUE(I)
!      write(*,*) 'ITYPE=1, ITRUE: ', ITRUE(I), 'CTRUE: ', CTRUE(I), 'ILEN: ', ILEN(I)
          IF (IOS.NE.0) ITRUE(I)=NINT(FERR)
        ELSE IF (ITYPE(I).EQ.2) THEN
          ITRUE(I)=IB36(CTRUE(I)(:ILEN(I)),NINT(FERR),ILEN(I))
        ELSE
          ITRUE(I)=ICHAR(CTRUE(I)(:1))
        ENDIF
!
        IF (ITRUE(I).EQ.NINT(FMISS)) THEN
          FTRUE(I)=FMISS
        ELSE IF (ITRUE(I).EQ.NINT(FERR)) THEN
          FTRUE(I)=FERR
        ELSE IF (ITYPE(I).LE.2) THEN
          FTRUE(I)=ITRUE(I)*FUNITS(I)
        ELSE
          FTRUE(I)=ITRUE(I)
        ENDIF
  190 CONTINUE
      RETURN
!-----------------------------------------------------------------------3456789
      ENTRY PUTRPT(RPT,CTRUE,ITRUE,FTRUE,FMISS,FERR                           &
     &,ILEN,ABBR,FMIN1,FUNITS,ITYPE,RPTID,NUM)
! CONVERT FLOATING POINT VALUES TO CHARACTERS
!
      STR=' '
      TRS=0
      DO 280 I=1,NUM
        IF (FTRUE(I).EQ.FMISS) THEN
          ITRUE(I)=NINT(FMISS)
        ELSE IF (FTRUE(I).EQ.FERR) THEN
          ITRUE(I)=NINT(FERR)
        ELSE IF (ITYPE(I).LE.2) THEN
          ITRUE(I)=NINT(FTRUE(I)/FUNITS(I))
        ELSE
          ITRUE(I)=NINT(FTRUE(I))
        ENDIF
!
        IF (ITRUE(I).EQ.NINT(FMISS)) THEN
          CTRUE(I)(:MIN(ILEN(I),LEN(CTRUE(I))))=' '
        ELSE IF (ITYPE(I).EQ.1) THEN
          WRITE(CTRUE(I)(:ILEN(I)),IFMT(ILEN(I)))ITRUE(I)
        ELSE IF (ITYPE(I).EQ.2) THEN
          CTRUE(I)(:ILEN(I))=B36(ITRUE(I),'*****',ILEN(I))
        ELSE
!          CTRUE(I)(:ILEN(I))=REPEAT(CHAR(ITRUE(I)),ILEN(I))
        ENDIF
!
        IF (ILEN(I).NE.1024) THEN
          IF (ILEN(I).GT.LEN(CTRUE(I))) STOP 'PUTRPT LEN CTRUE'
          STR(TRS+1:TRS+ILEN(I))=CTRUE(I)(:ILEN(I))
          TRS=TRS+ILEN(I)
        ENDIF
  280 CONTINUE
!
      RPT=' '
      PTR=0
      ATTC=0
      ATTI=0
      TRS=0
      DO 290 I=1,NUM
        IF (ABBR(I).EQ.' ATTI') THEN
          ATTI=NINT(FMIN1(I))
          ATTL=NINT(FMIN1(I+1))
          IF (ATTL.GT.99) THEN
            WRITE(STR(TRS+1:TRS+4),'(I2,A2)')ATTI,B36(ATTL,'**',2)
          ELSE
            WRITE(STR(TRS+1:TRS+4),'(2I2)')ATTI,ATTL
          ENDIF
          IF (ATTI.EQ.99) THEN
            IF (TRS+5.GT.LEN(STR)) STOP 'PUTRPT LEN STR'
            IF (NUM.EQ.I+3) THEN
              IF (CTRUE(NUM)(:).NE.' ') THEN
                ATTC=ATTC+1
                IF (PTR+6.GT.LEN(RPT)) STOP 'PUTRPT LEN RPT'
                RPT(PTR+1:PTR+5)=STR(TRS+1:TRS+5)
                PTR=PTR+5
                RPT(PTR+1:)=CTRUE(NUM)(:)
                IF (RPT(PTR+1:).NE.CTRUE(NUM)(:)) STOP 'PUTRPT LEN SUPD'
              ENDIF
            ENDIF
          ELSE
            IF (TRS+ATTL.GT.LEN(STR)) STOP 'PUTRPT LEN STR'
            IF (STR(TRS+5:TRS+ATTL).NE.' ') THEN
              ATTC=ATTC+1
              IF (PTR+ATTL.GT.LEN(RPT)) STOP 'PUTRPT LEN RPT'
              RPT(PTR+1:PTR+ATTL)=STR(TRS+1:TRS+ATTL)
              PTR=PTR+ATTL
            ENDIF
          ENDIF
        ELSE IF (ATTI.EQ.0) THEN
          IF (TRS+ILEN(I).GT.LEN(STR)) STOP 'PUTRPT LEN STR'
          IF (PTR+ILEN(I).GT.LEN(RPT)) STOP 'PUTRPT LEN RPT'
          IF (ABBR(I).EQ.'   IM') THEN
            STR(TRS+1:TRS+ILEN(I))=RPTID
          ELSE IF (ABBR(I).EQ.' ATTC') THEN
            STR(TRS+1:TRS+ILEN(I))='%'
          ENDIF
          RPT(PTR+1:PTR+ILEN(I))=STR(TRS+1:TRS+ILEN(I))
          PTR=PTR+ILEN(I)
        ENDIF
        TRS=TRS+ILEN(I)
  290 CONTINUE
      PTR=INDEX(RPT,'%')
      RPT(PTR:PTR)=B36(ATTC,'*',1)
      END
!-----------------------------------------------------------------------3456789
      SUBROUTINE MINMAX(CTRUE,ITRUE,FTRUE,FMISS,FERR                          &
     &,ILEN,FMIN1,FMAX1,FMIN2,FMAX2,FUNITS,ITYPE,NUM)
! SET EXTREME FLOATING POINT VALUES TO ERROR VALUE
      IMPLICIT INTEGER(A-E,G-Z)
      CHARACTER*(*) CTRUE
      DIMENSION CTRUE(NUM),ITRUE(NUM),FTRUE(NUM)                              &
     &,ILEN(NUM),FMIN1(NUM),FMAX1(NUM),FMIN2(NUM),FMAX2(NUM)                  &
     &,FUNITS(NUM),ITYPE(NUM)
!
      DO 190 I=1,NUM
        IF (ITYPE(I).LE.2) THEN
          IF (FTRUE(I).EQ.FMISS .OR. FTRUE(I).EQ.FERR) THEN
            ITRUE(I)=NINT(FTRUE(I))
            GOTO 190
          ENDIF
          ITRUE(I)=NINT(FTRUE(I)/FUNITS(I))
          IF (ITRUE(I).GE.NINT(FMIN1(I)/FUNITS(I))                            &
     &   .AND.ITRUE(I).LE.NINT(FMAX1(I)/FUNITS(I))                            &
     &   .OR. FMIN2(I).NE.FMISS                                               &
     &   .AND.ITRUE(I).GE.NINT(FMIN2(I)/FUNITS(I))                            &
     &   .AND.ITRUE(I).LE.NINT(FMAX2(I)/FUNITS(I))) THEN
          ELSE
            FTRUE(I)=FERR
            ITRUE(I)=NINT(FERR)
          ENDIF
        ELSE
          IF (ILEN(I).EQ.1024) THEN
            LENGTH=LEN(CTRUE(I))
          ELSE
            IF (ILEN(I).GT.LEN(CTRUE(I))) STOP 'MINMAX LEN CTRUE'
            LENGTH=ILEN(I)
          ENDIF
          IF (CTRUE(I)(:LENGTH).EQ.' ') THEN
            FTRUE(I)=FMISS
            ITRUE(I)=NINT(FMISS)
            GOTO 190
          ENDIF
          FTRUE(I)=ICHAR(CTRUE(I)(:1))
          ITRUE(I)=NINT(FTRUE(I))
          DO 180 J=1,LENGTH
            IF (LGE(CTRUE(I)(J:J),'A')                                        &
     &     .AND.LLE(CTRUE(I)(J:J),'Z')                                        &
     &     .OR. NINT(FMIN1(I)).LE.48                                          &
     &     .AND.LGE(CTRUE(I)(J:J),'0')                                        &
     &     .AND.LLE(CTRUE(I)(J:J),'9')                                        &
     &     .OR. NINT(FMIN1(I)).EQ.32                                          &
     &     .AND.LGE(CTRUE(I)(J:J),' ')                                        &
     &     .AND.LLE(CTRUE(I)(J:J),'~')) THEN
            ELSE
              FTRUE(I)=FERR
              ITRUE(I)=NINT(FERR)
              GOTO 190
            ENDIF
  180     CONTINUE
        ENDIF
  190 CONTINUE
      END
!-----------------------------------------------------------------------3456789
      SUBROUTINE SAVSUM(CTRUE,FTRUE,FMISS,FERR                                &
     &,ILEN,ABBR,NUM)
! SAVE SUMMARY INFORMATION
      IMPLICIT INTEGER(A-E,G-Z)
      CHARACTER*(*) PROGID,ABBR,CTRUE
      DIMENSION CTRUE(NUM),FTRUE(NUM),ILEN(NUM),ABBR(NUM)
      CHARACTER*64 STR
      DIMENSION SUM1(1024),SUM2(1024),SUM3(1024)
      DATA SUM1,SUM2,SUM3/1024*0,1024*0,1024*0/
      SAVE SUM1,SUM2,SUM3
      PC(A1,A2)=NINT(FLOAT(A1*100)/MAX(A2,1))
!
      DO 190 I=1,NUM
        IF (FTRUE(I).NE.FMISS .AND. FTRUE(I).NE.FERR) THEN
          SUM1(I)=SUM1(I)+1
        ELSE
          SUM2(I)=SUM2(I)+1
          IF (FTRUE(I).EQ.FMISS) THEN
            IF (ILEN(I).EQ.1024) THEN
              IF (CTRUE(I)(:).EQ.' ') GOTO 190
            ELSE
              IF (ILEN(I).GT.LEN(CTRUE(I))) STOP 'SAVSUM LEN CTRUE'
              IF (CTRUE(I)(:ILEN(I)).EQ.' ') GOTO 190
            ENDIF
          ENDIF
          SUM3(I)=SUM3(I)+1
          WRITE(STR,'(I3,1X,A5,2X,A53)')I,ABBR(I),CTRUE(I)
          CALL SAVSTR(STR)
        ENDIF
  190 CONTINUE
      RETURN
!-----------------------------------------------------------------------3456789
      ENTRY PRNSUM(UNIT,PROGID,ABBR,NUM)
! PRINT SUMMARY INFORMATION TO UNIT
!
      WRITE(UNIT,'(A)')PROGID
      WRITE(UNIT,'(/1X,A)')'SUMMARY OF FIELDS'
      WRITE(UNIT,'(A6,2(2X,A10,A11,A13))')'FIELD'                             &
     &,'# EXTANT','# MISSING','# ERRONEOUS'                                   &
     &,'% EXTANT','% MISSING','% ERRONEOUS'
      DO 290 I=1,NUM
        IF (SUM1(I).GT.0 .OR. SUM3(I).GT.0) THEN
          TOTAL=SUM1(I)+SUM2(I)
          WRITE(UNIT,'(1X,A5,2(2X,I10,I11,I13))')ABBR(I)                      &
     &    ,   SUM1(I)       ,   SUM2(I)       ,   SUM3(I)                     &
     &    ,PC(SUM1(I),TOTAL),PC(SUM2(I),TOTAL),PC(SUM3(I),TOTAL)
        ENDIF
        SUM1(I)=0
        SUM2(I)=0
        SUM3(I)=0
  290 CONTINUE
      WRITE(UNIT,'(/1X,A)')'SUMMARY OF ERRORS'
      WRITE(UNIT,'(A6,2X,A,T70,A)')'FIELD','ERROR','FREQUENCY'
      CALL PRNSTR(UNIT)
      END
!-----------------------------------------------------------------------3456789
      SUBROUTINE SAVSTR(STR)
! SAVE FREQUENCY OF STRING
      IMPLICIT INTEGER(A-E,G-Z)
      PARAMETER(MMAX=10000)
      CHARACTER*64 STR,ARR1
      DIMENSION ARR1(MMAX),ARR2(MMAX)
      DATA M/0/
      SAVE
!
      DO 190 I=1,M
        IF (STR.NE.ARR1(I)) GOTO 190
        ARR2(I)=ARR2(I)+1
        RETURN
  190 CONTINUE
      IF (M+1.GE.MMAX) STOP 'SAVSTR INCREASE MMAX'
      M=M+1
      ARR1(M)=STR
      ARR2(M)=1
      RETURN
!-----------------------------------------------------------------------3456789
      ENTRY PRNSTR(UNIT)
! PRINT FREQUENCY OF STRING
!
      DO 290 K=1,M-1
        J=K
        DO 280 L=K+1,M
          IF (LLT(ARR1(L),ARR1(J))) J=L
  280   CONTINUE
        IF (J.NE.K) THEN
          ARR1(MMAX)=ARR1(K)
          ARR2(MMAX)=ARR2(K)
          ARR1(K)=ARR1(J)
          ARR2(K)=ARR2(J)
          ARR1(J)=ARR1(MMAX)
          ARR2(J)=ARR2(MMAX)
        ENDIF
  290 CONTINUE
      WRITE(UNIT,'(A,T70,I9)')(ARR1(I)(4:),ARR2(I),I=1,M)
      M=0
      END
!-----------------------------------------------------------------------3456789
      FUNCTION IB36(B36,IERR,ILEN)
! CONVERT FROM BASE 36 CHARACTER STRING TO INTEGER
      IMPLICIT INTEGER (A-Z)
      CHARACTER (LEN=*) B36,STR
      PARAMETER(STR='0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ')
!
      IF (ILEN.GE.6) STOP 'IB36'
      IF (VERIFY(B36,STR).EQ.0) THEN
        IB36=0
        INT=1
        DO J=ILEN,1,-1
          IB36=IB36+(INDEX(STR,B36(J:J))-1)*INT
          INT=INT*36
        ENDDO
      ELSE
        IB36=IERR
      ENDIF
      END
!-----------------------------------------------------------------------3456789
      FUNCTION B36(IB36,ERR,ILEN)
! CONVERT FROM INTEGER TO BASE 36 CHARACTER STRING
      IMPLICIT INTEGER (A-Z)
      CHARACTER (LEN=*) B36,STR,ERR
      PARAMETER(STR='0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ')
!
      IF (ILEN.GE.6) STOP 'B36'
      IF (IB36.GE.0 .AND. IB36.LE.36**ILEN-1) THEN
        B36=' '
        INT=1
        DO J=ILEN,1,-1
          B36(J:J)=STR(MOD(IB36/INT,36)+1:)
          INT=INT*36
        ENDDO
      ELSE
        B36=ERR
      ENDIF
      END
!-----------------------------------------------------------------------3456789
      SUBROUTINE GETUID(CTRUE)
! GET UNIQUE REPORT ID
      CHARACTER*6 CTRUE
!
      OPEN(98,FILE='LAST_UID')
      READ(98,'(A)')CTRUE
      CALL B36INC(CTRUE)
      REWIND(98)
      WRITE(98,'(A)')CTRUE
      CLOSE(98)
      END
!-----------------------------------------------------------------------3456789
      SUBROUTINE B36INC(B36)
! INCREMENT BASE 36 CHARACTER STRING
      IMPLICIT INTEGER (A-Z)
      CHARACTER (LEN=*) B36,STR
      PARAMETER(STR='0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ')
!
      IF (VERIFY(B36,STR).EQ.0) THEN
        DO J=LEN(B36),1,-1
          NDX=INDEX(STR,B36(J:J))
          IF (NDX.LT.36) THEN
            B36(J:J)=STR(NDX+1:NDX+1)
            RETURN
          ELSE
            B36(J:J)=STR(1:1)
          ENDIF
        ENDDO
      ENDIF
      STOP 'B36INC'
      END
!-----------------------------------------------------------------------3456789
      SUBROUTINE EAST(ITRUE,FTRUE,FERR)
! CONVERT LONGITUDE TO DEGREES EAST
      IMPLICIT INTEGER(A-E,G-Z)
!
      ITRUE=NINT(FTRUE/0.01)
      IF(ITRUE.GE.-36000 .AND. ITRUE.LE.36000)THEN
        ITRUE=MOD(ITRUE+36000,36000)
        FTRUE=ITRUE*0.01
      ELSE
        ITRUE=NINT(FERR)
        FTRUE=FERR
      ENDIF
      END
!-----------------------------------------------------------------------3456789
      SUBROUTINE SDEG(ITRUE,FTRUE,FERR)
! CONVERT LONGITUDE TO SIGNED DEGREES EAST
      IMPLICIT INTEGER(A-E,G-Z)
!
      IF(NINT(FTRUE*100.).GE.-36000 .AND. NINT(FTRUE*100.).LE.36000)THEN
        IF(NINT(FTRUE*100.).GT.18000)FTRUE=FTRUE-360.
        ITRUE=NINT(FTRUE*100.)
      ELSE
        FTRUE=FERR
        ITRUE=NINT(FERR)
      ENDIF
      END
!-----------------------------------------------------------------------3456789
      SUBROUTINE ZEROS(CTRUE,ILEN,ABBR,ITYPE,NUM)
! ZERO FILL LEADING BLANKS
      IMPLICIT INTEGER(A-E,G-Z)
      CHARACTER*(*) CTRUE,ABBR
      DIMENSION CTRUE(NUM),ILEN(NUM),ABBR(NUM),ITYPE(NUM)
      CHARACTER IFMT(18)*8
      DATA IFMT/'(I1.1)','(I2.2)','(I3.3)','(I4.4)','(I5.5)','(I6.6)'         &
     &,'(I7.7)','(I8.8)','(I9.9)','(I1.0)','(I2.1)','(I3.2)','(I4.3)'         &
     &,'(I5.4)','(I6.5)','(I7.6)','(I8.7)','(I9.8)'/
      SAVE IFMT
!
      DO 190 I=1,NUM
        IF (ITYPE(I).EQ.1                                                     &
     & .AND. ILEN(I).NE.1                                                     &
     & .AND. ABBR(I).NE.'   IM'                                               &
     & .AND. ABBR(I).NE.' ATTI'                                               &
     & .AND. ABBR(I).NE.' ATTL') THEN
          IF (ILEN(I).GT.LEN(CTRUE(I))) STOP 'ZEROS LEN CTRUE'
          IF (CTRUE(I)(:ILEN(I)).EQ.' ') GOTO 190
          READ(CTRUE(I)(:ILEN(I)),IFMT(ILEN(I)))ITRUE
          IF (ITRUE.LT.0) THEN
            WRITE(CTRUE(I)(:ILEN(I)),IFMT(ILEN(I)+9))ITRUE
          ELSE
            WRITE(CTRUE(I)(:ILEN(I)),IFMT(ILEN(I)))ITRUE
          ENDIF
        ENDIF
  190 CONTINUE
      END
!-----------------------------------------------------------------------3456789
      BLOCK DATA BDIMMA
! COMMON BLOCK DATA STATEMENTS
      IMPLICIT INTEGER(A-E,G-Z)
!
!     missing = -999999
!     ILEN    = field length
!     ABBR    = field abbreviation
!     FMIN1   = field range minimum (first, or only)
!     FMAX1   = field range maximum (first, or only)
!     FMIN2   = field range minimum (second, or missing)
!     FMAX2   = field range maximum (second, or missing)
!     FUNITS  = field units (missing, if itype=3)
!     ITYPE   = 1: numeric:   decimal range(s)
!               2: numeric:   decimal range(s), transformed to base36
!               3: character: decimal range(s), transformed to ascii
!     RPTID   = 1 (IMMA version that the program can read)
!
      PARAMETER(NUM=245)
      CHARACTER*8 ABBR,RPTID
      COMMON /IMMA1/ILEN(NUM),ABBR(NUM),FMIN1(NUM),FMAX1(NUM)                 &
     &,FMIN2(NUM),FMAX2(NUM),FUNITS(NUM),ITYPE(NUM),RPTID
!
      DATA (ILEN(I),ABBR(I),FMIN1(I),FMAX1(I),FMIN2(I),FMAX2(I)               &
     &,FUNITS(I),ITYPE(I),I=1,18)/                                            &
     &   4,'   YR', 1600.  , 2024.  ,-999999.,-999999., 1.     , 1,           &
     &   2,'   MO',    1.  ,   12.  ,-999999.,-999999., 1.     , 1,           &
     &   2,'   DY',    1.  ,   31.  ,-999999.,-999999., 1.     , 1,           &
     &   4,'   HR',    0.00,   23.99,-999999.,-999999., 0.01   , 1,           &
     &   5,'  LAT',  -90.00,   90.00,-999999.,-999999., 0.01   , 1,           &
     &   6,'  LON',    0.00,  359.99, -179.99,  180.00, 0.01   , 1,           &
     &   2,'   IM',    0.  ,   99.  ,-999999.,-999999., 1.     , 1,           &
     &   1,' ATTC',    0.  ,    9.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'   TI',    0.  ,    3.  ,-999999.,-999999., 1.     , 1,           &
     &   1,'   LI',    0.  ,    6.  ,-999999.,-999999., 1.     , 1,           &
     &   1,'   DS',    0.  ,    9.  ,-999999.,-999999., 1.     , 1,           &
     &   1,'   VS',    0.  ,    9.  ,-999999.,-999999., 1.     , 1,           &
     &   2,'  NID',    0.  ,   99.  ,-999999.,-999999., 1.     , 1,           &
     &   2,'   II',    0.  ,   10.  ,-999999.,-999999., 1.     , 1,           &
     &   9,'   ID',   32.  ,  126.  ,-999999.,-999999.,-999999., 3,           &
     &   2,'   C1',   48.  ,   57.  ,   65.  ,   90.  ,-999999., 3,           &
     &   1,'   DI',    0.  ,    6.  ,-999999.,-999999., 1.     , 1,           &
     &   3,'    D',    1.  ,  362.  ,-999999.,-999999., 1.     , 1/
      DATA (ILEN(I),ABBR(I),FMIN1(I),FMAX1(I),FMIN2(I),FMAX2(I)               &
     &,FUNITS(I),ITYPE(I),I=19,36)/                                           &
     &   1,'   WI',    0.  ,    8.  ,-999999.,-999999., 1.     , 1,           &
     &   3,'    W',    0.0 ,   99.9 ,-999999.,-999999., 0.1    , 1,           &
     &   1,'   VI',    0.  ,    2.  ,-999999.,-999999., 1.     , 1,           &
     &   2,'   VV',   90.  ,   99.  ,-999999.,-999999., 1.     , 1,           &
     &   2,'   WW',    0.  ,   99.  ,-999999.,-999999., 1.     , 1,           &
     &   1,'   W1',    0.  ,    9.  ,-999999.,-999999., 1.     , 1,           &
     &   5,'  SLP',  870.0 , 1074.6 ,-999999.,-999999., 0.1    , 1,           &
     &   1,'    A',    0.  ,    8.  ,-999999.,-999999., 1.     , 1,           &
     &   3,'  PPP',    0.0 ,   51.0 ,-999999.,-999999., 0.1    , 1,           &
     &   1,'   IT',    0.  ,    9.  ,-999999.,-999999., 1.     , 1,           &
     &   4,'   AT',  -99.9 ,   99.9 ,-999999.,-999999., 0.1    , 1,           &
     &   1,' WBTI',    0.  ,    3.  ,-999999.,-999999., 1.     , 1,           &
     &   4,'  WBT',  -99.9 ,   99.9 ,-999999.,-999999., 0.1    , 1,           &
     &   1,' DPTI',    0.  ,    3.  ,-999999.,-999999., 1.     , 1,           &
     &   4,'  DPT',  -99.9 ,   99.9 ,-999999.,-999999., 0.1    , 1,           &
     &   2,'   SI',    0.  ,   12.  ,-999999.,-999999., 1.     , 1,           &
     &   4,'  SST',  -99.9 ,   99.9 ,-999999.,-999999., 0.1    , 1,           &
     &   1,'    N',    0.  ,    9.  ,-999999.,-999999., 1.     , 1/
      DATA (ILEN(I),ABBR(I),FMIN1(I),FMAX1(I),FMIN2(I),FMAX2(I)               &
     &,FUNITS(I),ITYPE(I),I=37,54)/                                           &
     &   1,'   NH',    0.  ,    9.  ,-999999.,-999999., 1.     , 1,           &
     &   1,'   CL',    0.  ,   10.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'   HI',    0.  ,    1.  ,-999999.,-999999., 1.     , 1,           &
     &   1,'    H',    0.  ,   10.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'   CM',    0.  ,   10.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'   CH',    0.  ,   10.  ,-999999.,-999999., 1.     , 2,           &
     &   2,'   WD',    0.  ,   38.  ,-999999.,-999999., 1.     , 1,           &
     &   2,'   WP',    0.  ,   30.  ,   99.  ,   99.  , 1.     , 1,           &
     &   2,'   WH',    0.  ,   99.  ,-999999.,-999999., 1.     , 1,           &
     &   2,'   SD',    0.  ,   38.  ,-999999.,-999999., 1.     , 1,           &
     &   2,'   SP',    0.  ,   30.  ,   99.  ,   99.  , 1.     , 1,           &
     &   2,'   SH',    0.  ,   99.  ,-999999.,-999999., 1.     , 1,           &
     &   2,' ATTI',    1.  ,    1.  ,    1.  ,    1.  , 1.     , 1,           &
     &   2,' ATTL',   65.  ,   65.  ,   65.  ,   65.  , 1.     , 1,           &
     &   1,'  BSI',-999999.,-999999.,-999999.,-999999., 1.     , 1,           &
     &   3,'  B10',    1.  ,  648.  ,-999999.,-999999., 1.     , 1,           &
     &   2,'   B1',    0.  ,   99.  ,-999999.,-999999., 1.     , 1,           &
     &   3,'  DCK',    0.  ,  999.  ,-999999.,-999999., 1.     , 1/
      DATA (ILEN(I),ABBR(I),FMIN1(I),FMAX1(I),FMIN2(I),FMAX2(I)               &
     &,FUNITS(I),ITYPE(I),I=55,72)/                                           &
     &   3,'  SID',    0.  ,  999.  ,-999999.,-999999., 1.     , 1,           &
     &   2,'   PT',    0.  ,   21.  ,-999999.,-999999., 1.     , 1,           &
     &   2,' DUPS',    0.  ,   14.  ,-999999.,-999999., 1.     , 1,           &
     &   1,' DUPC',    0.  ,    2.  ,-999999.,-999999., 1.     , 1,           &
     &   1,'   TC',    0.  ,    1.  ,-999999.,-999999., 1.     , 1,           &
     &   1,'   PB',    0.  ,    2.  ,-999999.,-999999., 1.     , 1,           &
     &   1,'   WX',    1.  ,    1.  ,-999999.,-999999., 1.     , 1,           &
     &   1,'   SX',    1.  ,    1.  ,-999999.,-999999., 1.     , 1,           &
     &   2,'   C2',    0.  ,   40.  ,-999999.,-999999., 1.     , 1,           &
     &   1,'  SQZ',    1.  ,   35.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'  SQA',    1.  ,   21.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'  AQZ',    1.  ,   35.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'  AQA',    1.  ,   21.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'  UQZ',    1.  ,   35.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'  UQA',    1.  ,   21.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'  VQZ',    1.  ,   35.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'  VQA',    1.  ,   21.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'  PQZ',    1.  ,   35.  ,-999999.,-999999., 1.     , 2/
      DATA (ILEN(I),ABBR(I),FMIN1(I),FMAX1(I),FMIN2(I),FMAX2(I)               &
     &,FUNITS(I),ITYPE(I),I=73,90)/                                           &
     &   1,'  PQA',    1.  ,   21.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'  DQZ',    1.  ,   35.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'  DQA',    1.  ,   21.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'   ND',    1.  ,    2.  ,-999999.,-999999., 1.     , 1,           &
     &   1,'   SF',    1.  ,   15.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'   AF',    1.  ,   15.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'   UF',    1.  ,   15.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'   VF',    1.  ,   15.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'   PF',    1.  ,   15.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'   RF',    1.  ,   15.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'  ZNC',    1.  ,   10.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'  WNC',    1.  ,   10.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'  BNC',    1.  ,   10.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'  XNC',    1.  ,   10.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'  YNC',    1.  ,   10.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'  PNC',    1.  ,   10.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'  ANC',    1.  ,   10.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'  GNC',    1.  ,   10.  ,-999999.,-999999., 1.     , 2/
      DATA (ILEN(I),ABBR(I),FMIN1(I),FMAX1(I),FMIN2(I),FMAX2(I)               &
     &,FUNITS(I),ITYPE(I),I=91,108)/                                          &
     &   1,'  DNC',    1.  ,   10.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'  SNC',    1.  ,   10.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'  CNC',    1.  ,   10.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'  ENC',    1.  ,   10.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'  FNC',    1.  ,   10.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'  TNC',    1.  ,   10.  ,-999999.,-999999., 1.     , 2,           &
     &   2,'  QCE',    0.  ,   63.  ,-999999.,-999999., 1.     , 1,           &
     &   1,'   LZ',    1.  ,    1.  ,-999999.,-999999., 1.     , 1,           &
     &   2,'  QCZ',    0.  ,   31.  ,-999999.,-999999., 1.     , 1,           &
     &   2,' ATTI',    5.  ,    5.  ,    5.  ,    5.  , 1.     , 1,           &
     &   2,' ATTL',   94.  ,   94.  ,   94.  ,   94.  , 1.     , 1,           &
     &   1,'   OS',    0.  ,    6.  ,-999999.,-999999., 1.     , 1,           &
     &   1,'   OP',    0.  ,    9.  ,-999999.,-999999., 1.     , 1,           &
     &   1,'   FM',    0.  ,   35.  ,-999999.,-999999., 1.     , 2,           &
     &   1,' IMMV',    0.  ,   35.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'   IX',    1.  ,    7.  ,-999999.,-999999., 1.     , 1,           &
     &   1,'   W2',    0.  ,    9.  ,-999999.,-999999., 1.     , 1,           &
     &   1,'  WMI',    0.  ,    9.  ,-999999.,-999999., 1.     , 1/
      DATA (ILEN(I),ABBR(I),FMIN1(I),FMAX1(I),FMIN2(I),FMAX2(I)               &
     &,FUNITS(I),ITYPE(I),I=109,126)/                                         &
     &   2,'  SD2',    0.  ,   38.  ,-999999.,-999999., 1.     , 1,           &
     &   2,'  SP2',    0.  ,   30.  ,   99.  ,   99.  , 1.     , 1,           &
     &   2,'  SH2',    0.  ,   99.  ,-999999.,-999999., 1.     , 1,           &
     &   1,'   IS',    1.  ,    5.  ,-999999.,-999999., 1.     , 1,           &
     &   2,'   ES',    0.  ,   99.  ,-999999.,-999999., 1.     , 1,           &
     &   1,'   RS',    0.  ,    4.  ,-999999.,-999999., 1.     , 1,           &
     &   1,'  IC1',    0.  ,   10.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'  IC2',    0.  ,   10.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'  IC3',    0.  ,   10.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'  IC4',    0.  ,   10.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'  IC5',    0.  ,   10.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'   IR',    0.  ,    4.  ,-999999.,-999999., 1.     , 1,           &
     &   3,'  RRR',    0.  ,  999.  ,-999999.,-999999., 1.     , 1,           &
     &   1,'   TR',    1.  ,    9.  ,-999999.,-999999., 1.     , 1,           &
     &   1,'   NU',   32.  ,  126.  ,-999999.,-999999.,-999999., 3,           &
     &   1,'  QCI',    0.  ,    9.  ,-999999.,-999999., 1.     , 1,           &
     &   1,'  QI1',    0.  ,    9.  ,-999999.,-999999., 1.     , 1,           &
     &   1,'  QI2',    0.  ,    9.  ,-999999.,-999999., 1.     , 1/
      DATA (ILEN(I),ABBR(I),FMIN1(I),FMAX1(I),FMIN2(I),FMAX2(I)               &
     &,FUNITS(I),ITYPE(I),I=127,144)/                                         &
     &   1,'  QI3',    0.  ,    9.  ,-999999.,-999999., 1.     , 1,           &
     &   1,'  QI4',    0.  ,    9.  ,-999999.,-999999., 1.     , 1,           &
     &   1,'  QI5',    0.  ,    9.  ,-999999.,-999999., 1.     , 1,           &
     &   1,'  QI6',    0.  ,    9.  ,-999999.,-999999., 1.     , 1,           &
     &   1,'  QI7',    0.  ,    9.  ,-999999.,-999999., 1.     , 1,           &
     &   1,'  QI8',    0.  ,    9.  ,-999999.,-999999., 1.     , 1,           &
     &   1,'  QI9',    0.  ,    9.  ,-999999.,-999999., 1.     , 1,           &
     &   1,' QI10',    0.  ,    9.  ,-999999.,-999999., 1.     , 1,           &
     &   1,' QI11',    0.  ,    9.  ,-999999.,-999999., 1.     , 1,           &
     &   1,' QI12',    0.  ,    9.  ,-999999.,-999999., 1.     , 1,           &
     &   1,' QI13',    0.  ,    9.  ,-999999.,-999999., 1.     , 1,           &
     &   1,' QI14',    0.  ,    9.  ,-999999.,-999999., 1.     , 1,           &
     &   1,' QI15',    0.  ,    9.  ,-999999.,-999999., 1.     , 1,           &
     &   1,' QI16',    0.  ,    9.  ,-999999.,-999999., 1.     , 1,           &
     &   1,' QI17',    0.  ,    9.  ,-999999.,-999999., 1.     , 1,           &
     &   1,' QI18',    0.  ,    9.  ,-999999.,-999999., 1.     , 1,           &
     &   1,' QI19',    0.  ,    9.  ,-999999.,-999999., 1.     , 1,           &
     &   1,' QI20',    0.  ,    9.  ,-999999.,-999999., 1.     , 1/
      DATA (ILEN(I),ABBR(I),FMIN1(I),FMAX1(I),FMIN2(I),FMAX2(I)               &
     &,FUNITS(I),ITYPE(I),I=145,162)/                                         &
     &   1,' QI21',    0.  ,    9.  ,-999999.,-999999., 1.     , 1,           &
     &   3,'  HDG',    0.  ,  360.  ,-999999.,-999999., 1.     , 1,           &
     &   3,'  COG',    0.  ,  360.  ,-999999.,-999999., 1.     , 1,           &
     &   2,'  SOG',    0.  ,   99.  ,-999999.,-999999., 1.     , 1,           &
     &   2,'  SLL',    0.  ,   99.  ,-999999.,-999999., 1.     , 1,           &
     &   3,' SLHH',  -99.  ,   99.  ,-999999.,-999999., 1.     , 1,           &
     &   3,'  RWD',    1.  ,  362.  ,-999999.,-999999., 1.     , 1,           &
     &   3,'  RWS',    0.0 ,   99.9 ,-999999.,-999999., 0.1    , 1,           &
     &   1,' QI22',    0.  ,    9.  ,-999999.,-999999., 1.     , 1,           &
     &   1,' QI23',    0.  ,    9.  ,-999999.,-999999., 1.     , 1,           &
     &   1,' QI24',    0.  ,    9.  ,-999999.,-999999., 1.     , 1,           &
     &   1,' QI25',    0.  ,    9.  ,-999999.,-999999., 1.     , 1,           &
     &   1,' QI26',    0.  ,    9.  ,-999999.,-999999., 1.     , 1,           &
     &   1,' QI27',    0.  ,    9.  ,-999999.,-999999., 1.     , 1,           &
     &   1,' QI28',    0.  ,    9.  ,-999999.,-999999., 1.     , 1,           &
     &   1,' QI29',    0.  ,    9.  ,-999999.,-999999., 1.     , 1,           &
     &   4,'   RH',    0.0 ,  100.0 ,-999999.,-999999., 0.1    , 1,           &
     &   1,'  RHI',    0.  ,    4.  ,-999999.,-999999., 1.     , 1/
      DATA (ILEN(I),ABBR(I),FMIN1(I),FMAX1(I),FMIN2(I),FMAX2(I)               &
     &,FUNITS(I),ITYPE(I),I=163,180)/                                         &
     &   1,' AWSI',    0.  ,    2.  ,-999999.,-999999., 1.     , 1,           &
     &   7,'IMONO',    0.  ,9999999.,-999999.,-999999., 1.     , 1,           &
     &   2,' ATTI',    6.  ,    6.  ,    6.  ,    6.  , 1.     , 1,           &
     &   2,' ATTL',   68.  ,   68.  ,   68.  ,   68.  , 1.     , 1,           &
     &   4,' CCCC',   65.  ,   90.  ,-999999.,-999999.,-999999., 3,           &
     &   6,' BUID',   48.  ,   57.  ,   65.  ,   90.  ,-999999., 3,           &
     &   1,'FBSRC',    0.  ,   35.  ,-999999.,-999999., 1.     , 2,           &
     &   5,'  BMP',  870.0 , 1074.6 ,-999999.,-999999., 0.1    , 1,           &
     &   4,' BSWU',  -99.9 ,   99.9 ,-999999.,-999999., 0.1    , 1,           &
     &   4,'  SWU',  -99.9 ,   99.9 ,-999999.,-999999., 0.1    , 1,           &
     &   4,' BSWV',  -99.9 ,   99.9 ,-999999.,-999999., 0.1    , 1,           &
     &   4,'  SWV',  -99.9 ,   99.9 ,-999999.,-999999., 0.1    , 1,           &
     &   4,' BSAT',  -99.9 ,   99.9 ,-999999.,-999999., 0.1    , 1,           &
     &   3,' BSRH',    0.  ,  100.  ,-999999.,-999999., 1.     , 1,           &
     &   3,'  SRH',    0.  ,  100.  ,-999999.,-999999., 1.     , 1,           &
     &   5,' BSST', -99.99 ,  99.99 ,-999999.,-999999., 0.01   , 1,           &
     &   1,'  MST',    0.  ,    9.  ,-999999.,-999999., 1.     , 1,           &
     &   4,'  MSH', -999.  , 9999.  ,-999999.,-999999., 1.     , 1/
      DATA (ILEN(I),ABBR(I),FMIN1(I),FMAX1(I),FMIN2(I),FMAX2(I)               &
     &,FUNITS(I),ITYPE(I),I=181,198)/                                         &
     &   4,'   BY',    0.  , 9999.  ,-999999.,-999999., 1.     , 1,           &
     &   2,'   BM',    1.  ,   12.  ,-999999.,-999999., 1.     , 1,           &
     &   2,'   BD',    1.  ,   31.  ,-999999.,-999999., 1.     , 1,           &
     &   2,'   BH',    0.  ,   23.  ,-999999.,-999999., 1.     , 1,           &
     &   2,'  BFL',    0.  ,   99.  ,-999999.,-999999., 1.     , 1,           &
     &   2,' ATTI',    7.  ,    7.  ,    7.  ,    7.  , 1.     , 1,           &
     &   2,' ATTL',   58.  ,   58.  ,   58.  ,   58.  , 1.     , 1,           &
     &   1,'  MDS',    0.  ,   35.  ,-999999.,-999999., 1.     , 2,           &
     &   2,'  C1M',   65.  ,   90.  ,-999999.,-999999.,-999999., 3,           &
     &   2,'  OPM',    0.  ,   99.  ,-999999.,-999999., 1.     , 1,           &
     &   2,'  KOV',   32.  ,  126.  ,-999999.,-999999.,-999999., 3,           &
     &   2,'  COR',   65.  ,   90.  ,-999999.,-999999.,-999999., 3,           &
     &   3,'  TOB',   32.  ,  126.  ,-999999.,-999999.,-999999., 3,           &
     &   3,'  TOT',   32.  ,  126.  ,-999999.,-999999.,-999999., 3,           &
     &   2,'  EOT',   32.  ,  126.  ,-999999.,-999999.,-999999., 3,           &
     &   2,'  LOT',   32.  ,  126.  ,-999999.,-999999.,-999999., 3,           &
     &   1,'  TOH',   32.  ,  126.  ,-999999.,-999999.,-999999., 3,           &
     &   2,'  EOH',   32.  ,  126.  ,-999999.,-999999.,-999999., 3/
      DATA (ILEN(I),ABBR(I),FMIN1(I),FMAX1(I),FMIN2(I),FMAX2(I)               &
     &,FUNITS(I),ITYPE(I),I=199,216)/                                         &
     &   3,'  SIM',   32.  ,  126.  ,-999999.,-999999.,-999999., 3,           &
     &   3,'  LOV',    0.  ,  999.  ,-999999.,-999999., 1.     , 1,           &
     &   2,'  DOS',    0.  ,   99.  ,-999999.,-999999., 1.     , 1,           &
     &   3,'  HOP',    0.  ,  999.  ,-999999.,-999999., 1.     , 1,           &
     &   3,'  HOT',    0.  ,  999.  ,-999999.,-999999., 1.     , 1,           &
     &   3,'  HOB',    0.  ,  999.  ,-999999.,-999999., 1.     , 1,           &
     &   3,'  HOA',    0.  ,  999.  ,-999999.,-999999., 1.     , 1,           &
     &   5,'  SMF',    0.  ,99999.  ,-999999.,-999999., 1.     , 1,           &
     &   5,'  SME',    0.  ,99999.  ,-999999.,-999999., 1.     , 1,           &
     &   2,'  SMV',    0.  ,   99.  ,-999999.,-999999., 1.     , 1,           &
     &   2,' ATTI',    8.  ,    8.  ,    8.  ,    8.  , 1.     , 1,           &
     &   2,' ATTL',  104.  ,  104.  ,  104.  ,  104.  , 1.     , 2,           &
     &   5,'  OTV',  -3.000,  35.999,-999999.,-999999., 0.001  , 1,           &
     &   4,'  OTZ',    0.00,   10.99,-999999.,-999999., 0.01   , 1,           &
     &   5,'  OSV',   0.000,  40.999,-999999.,-999999., 0.001  , 1,           &
     &   4,'  OSZ',    0.00,   10.99,-999999.,-999999., 0.01   , 1,           &
     &   4,'  OOV',    0.00,   12.99,-999999.,-999999., 0.01   , 1,           &
     &   4,'  OOZ',    0.00,   10.99,-999999.,-999999., 0.01   , 1/
      DATA (ILEN(I),ABBR(I),FMIN1(I),FMAX1(I),FMIN2(I),FMAX2(I)               &
     &,FUNITS(I),ITYPE(I),I=217,234)/                                         &
     &   4,'  OPV',    0.00,   30.99,-999999.,-999999., 0.01   , 1,           &
     &   4,'  OPZ',    0.00,   10.99,-999999.,-999999., 0.01   , 1,           &
     &   5,' OSIV',    0.00,  250.99,-999999.,-999999., 0.01   , 1,           &
     &   4,' OSIZ',    0.00,   10.99,-999999.,-999999., 0.01   , 1,           &
     &   5,'  ONV',    0.00,  500.99,-999999.,-999999., 0.01   , 1,           &
     &   4,'  ONZ',    0.00,   10.99,-999999.,-999999., 0.01   , 1,           &
     &   3,' OPHV',    6.20,    9.20,-999999.,-999999., 0.01   , 1,           &
     &   4,' OPHZ',    0.00,   10.99,-999999.,-999999., 0.01   , 1,           &
     &   4,'  OCV',    0.00,   50.99,-999999.,-999999., 0.01   , 1,           &
     &   4,'  OCZ',    0.00,   10.99,-999999.,-999999., 0.01   , 1,           &
     &   3,'  OAV',    0.00,    3.10,-999999.,-999999., 0.01   , 1,           &
     &   4,'  OAZ',    0.00,   10.00,-999999.,-999999., 0.01   , 1,           &
     &   4,' OPCV',    0.  , 9999.  ,-999999.,-999999., 1.     , 1,           &
     &   4,' OPCZ',    0.00,   10.99,-999999.,-999999., 0.01   , 1,           &
     &   4,'  ODV',    0.  , 9999.  ,-999999.,-999999., 1.     , 1,           &
     &   4,'  ODZ',    0.00,   10.99,-999999.,-999999., 0.01   , 1,           &
     &  10,' PUID',   48.  ,   57.  ,   65.  ,   90.  ,-999999., 3,           &
     &   2,' ATTI',   98.  ,   98.  ,   98.  ,   98.  , 1.     , 1/
      DATA (ILEN(I),ABBR(I),FMIN1(I),FMAX1(I),FMIN2(I),FMAX2(I)               &
     &,FUNITS(I),ITYPE(I),I=235,245)/                                         &
     &   2,' ATTL',   15.  ,   15.  ,   15.  ,   15.  , 1.     , 1,           &
     &   6,'  UID',   48.  ,   57.  ,   65.  ,   90.  ,-999999., 3,           &
     &   1,'  RN1',    0.  ,   35.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'  RN2',    0.  ,   35.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'  RN3',    0.  ,   35.  ,-999999.,-999999., 1.     , 2,           &
     &   1,'  RSA',    0.  ,    2.  ,-999999.,-999999., 1.     , 1,           &
     &   1,'  IRF',    0.  ,    1.  ,-999999.,-999999., 1.     , 1,           &
     &   2,' ATTI',   99.  ,   99.  ,   99.  ,   99.  , 1.     , 1,           &
     &   2,' ATTL',    0.  ,    0.  ,    0.  ,    0.  , 1.     , 1,           &
     &   1,' ATTE',    1.  ,    1.  ,-999999.,-999999., 1.     , 1,           &
     &1024,' SUPD',   32.  ,  126.  ,-999999.,-999999.,-999999., 3/
      DATA RPTID/' 1'/
      END
!-----------------------------------------------------------------------3456789
      real function fxktms(kt)
!-----Convert from knots (kt; with respect to the international nautical
!     mile) to meters per second (see {tpktms} for details).
!-----Adapted from colib5s.01J function {cvskm} (1984); sdw, 26 Jun 1998.
      real kt
      fxktms = kt * 0.51444444444444444444
      return
      end

EOR
rm a.out
lf95 p.f90
date
cat ./USNavy_marine_metar/NAVSHP-MTR-CLEAN.txt | ./a.out

